# Kept for backward compatibility
RC_DEFAULT_TEMPLATE = """\
# enstaller configuration file
# ============================
#
# This file contains the default package repositories and configuration
# used by enstaller %(version)s for the Python %(py_ver)s environment:
#
#   sys.prefix = %(sys_prefix)r
#
# This file was initially created by running the enpkg command.

%(auth_section)s

# `use_webservice` refers to using 'https://api.enthought.com/eggs/'.
# The default is True; that is, the webservice URL is used for fetching
# eggs.  Uncommenting changes this behavior to using the explicit
# IndexedRepos listed below.
#use_webservice = False

# When use_webservice is True, one can control the store entry point enpkg will
# talk to. If not specified, a default will be used. Mostly useful for testing
#store_url = "https://acme.com"

# The enpkg command searches for eggs in the list `IndexedRepos` defined
# below.  When enpkg searches for an egg, it tries each repository in
# this list in order and selects the first one that matches, ignoring
# remaining repositories.  Therefore, the order of this list matters.
#
# For local repositories, the index file is optional.  Remember that on
# Windows systems backslashes in a directory path need to escaped, e.g.:
# r'file://C:\\repository\\' or 'file://C:\\\\repository\\\\'
IndexedRepos = [
#  'https://www.enthought.com/repo/ets/eggs/{SUBDIR}/',
  'https://www.enthought.com/repo/epd/GPL-eggs/{SUBDIR}/',
  'https://www.enthought.com/repo/epd/eggs/{SUBDIR}/',
# The Enthought PyPI build mirror:
  'http://www.enthought.com/repo/pypi/eggs/{SUBDIR}/',
]

# Install prefix (enpkg --prefix and --sys-prefix options overwrite
# this).  When this variable is not provided, it will default to the
# value of sys.prefix (within the current interpreter running enpkg).
#prefix = %(sys_prefix)r

# When running enpkg behind a firewall it might be necessary to use a
# proxy to access the repositories.  The URL for the proxy can be set
# here.  Note that the enpkg --proxy option will overwrite this setting.
%(proxy_line)s

# Uncomment the next line to disable application menu-item installation.
# This only affects the few packages that install menu items, such as
# IPython.
#noapp = True

# Uncomment the next line to turn off automatic prompts to update
# enstaller.
#autoupdate = False

# Uncomment to disable pypi eggs
#use_pypi = False
"""

RC_TEMPLATE = """\
# enstaller configuration file
# ============================
#
# This file contains the default package repositories and configuration
# used by enstaller %(version)s for the Python %(py_ver)s environment:
#
#   sys.prefix = %(sys_prefix)r
#
# This file was initially created by running the enpkg command.

%(auth_section)s

# `use_webservice` refers to using 'https://api.enthought.com/eggs/'.
# The default is True; that is, the webservice URL is used for fetching
# eggs.  Uncommenting changes this behavior to using the explicit
# IndexedRepos listed below.
use_webservice = %(use_webservice)s

# When use_webservice is True, one can control the store entry point enpkg will
# talk to. If not specified, a default will be used. Mostly useful for testing
store_url = "%(store_url)s"

# The enpkg command searches for eggs in the list `IndexedRepos` defined
# below.  When enpkg searches for an egg, it tries each repository in
# this list in order and selects the first one that matches, ignoring
# remaining repositories.  Therefore, the order of this list matters.
#
# For local repositories, the index file is optional.  Remember that on
# Windows systems backslashes in a directory path need to escaped, e.g.:
# r'file://C:\\repository\\' or 'file://C:\\\\repository\\\\'
IndexedRepos = [
%(indexed_repositories)s
]

# Install prefix (enpkg --prefix and --sys-prefix options overwrite
# this).  When this variable is not provided, it will default to the
# value of sys.prefix (within the current interpreter running enpkg).
prefix = %(sys_prefix)r

# When running enpkg behind a firewall it might be necessary to use a
# proxy to access the repositories.  The URL for the proxy can be set
# here.  Note that the enpkg --proxy option will overwrite this setting.
%(proxy_line)s

# Uncomment the next line to disable application menu-item installation.
# This only affects the few packages that install menu items, such as
# IPython.
#noapp = True

# Uncomment the next line to turn off automatic prompts to update
# enstaller.
#autoupdate = False

# Whether to consider pypi eggs
use_pypi = %(use_pypi)s
"""



# Changes library path in object code (ELF and Mach-O).
from __future__ import print_function

import logging
import sys
import re
from os.path import abspath, join, islink, isfile, exists

logger = logging.getLogger(__name__)

# extensions which are assumed to belong to files which don't contain
# object code
NO_OBJ = ('.py', '.pyc', '.pyo', '.h', '.a', '.c', '.txt', '.html', '.xml',
          '.png', '.jpg', '.gif')

MAGIC = {
    b'\xca\xfe\xba\xbe': 'MachO-universal',
    b'\xce\xfa\xed\xfe': 'MachO-i386',
    b'\xcf\xfa\xed\xfe': 'MachO-x86_64',
    b'\xfe\xed\xfa\xce': 'MachO-ppc',
    b'\xfe\xed\xfa\xcf': 'MachO-ppc64',
    b'\x7fELF': 'ELF',
}

PLACEHOLD_PAT = re.compile(5 * b'/PLACEHOLD' + b'([^\0\\s]*)\0')


def get_object_type(path):
    """
    Return the object file type of the specified file (not link).
    Otherwise, if the file is not an object file, returns None.
    """
    if path.endswith(NO_OBJ) or islink(path) or not isfile(path):
        return None
    with open(path, 'rb') as fi:
        head = fi.read(4)
    return MAGIC.get(head)


def _find_lib(fn, targets):
    for tgt in targets:
        dst = abspath(join(tgt, fn))
        if exists(dst):
            return dst
    logger.error("ERROR: library %r not found", fn)
    return join('/ERROR/path/not/found', fn)


def _fix_object_code(path, targets):
    tp = get_object_type(path)
    if tp is None:
        return

    with open(path, 'r+b') as f:
        data = f.read()
        matches = list(PLACEHOLD_PAT.finditer(data))
        if not matches:
            return

        logger.info("Fixing placeholders in: %r", path)
        for m in matches:
            rest = m.group(1)
            original_r = rest
            while rest.startswith(b'/PLACEHOLD'):
                rest = rest[10:]

            if tp.startswith('MachO-') and rest.startswith(b'/'):
                # If the /PLACEHOLD is found in a LC_LOAD_DYLIB command
                r = _find_lib(rest[1:].decode("utf8"), targets).encode("utf8")
            else:
                # If the /PLACEHOLD is found in a LC_RPATH command (Mach-O) or in
                # R(UN)PATH on ELF
                assert rest == b'' or rest.startswith(b':')
                rpaths = list(target.encode("utf8") for target in targets)
                # extend the list with rpath which were already in the binary,
                # if any
                rpaths.extend(p for p in rest.split(b':') if p)
                r = b':'.join(rpaths)

            logger.info("replacing rpath %r with %r", original_r, r)

            padding = len(m.group(0)) - len(r)
            if padding < 1:  # we need at least one null-character
                raise Exception("placeholder %r too short" % m.group(0))
            r += padding * b'\0'
            assert m.start() + len(r) == m.end()
            f.seek(m.start())
            f.write(r)


def _compute_targets(egg_targets, prefix):
    """
    Compute the list of target direcories where shared object files are found
    """
    prefixes = [prefix] if prefix != abspath(sys.prefix) else [sys.prefix]

    targets = []
    for prefix in prefixes:
        for target in egg_targets:
            targets.append(join(prefix, target))
        targets.append(join(prefix, 'lib'))

    logger.info('Target directories:')
    for tgt in targets:
        logger.info('    %r', tgt)

    return targets


def apply_placeholder_hack(files, egg_targets, prefix):
    """
    Tries to fix the library path for all object files installed by the egg.
    """
    targets = _compute_targets(egg_targets, prefix)

    for path in files:
        _fix_object_code(path, targets)


# XXX: kept for legacy reason, DO NOT REMOVE.
# This code is used during enstaller inplace update.
def fix_files(egg_inst):
    if egg_inst.cname != "enstaller":
        msg = "Legacy dummy code used for an egg not called enstaller !"
        raise RuntimeError(msg)



from __future__ import absolute_import

from ._impl import APITokenAuth, UserPasswordAuth, subscription_message
from .user_info import DUMMY_USER, UserInfo

_INDEX_NAME = "index.json"

__all__ = ["subscription_message", "DUMMY_USER", "UserInfo"]
__all__ += ["APITokenAuth", "UserPasswordAuth"]



# -*- test-case-name: twisted.test.test_reflect -*-
# Copyright (c) Twisted Matrix Laboratories.
# See LICENSE for details.

"""
Standardized versions of various cool and/or strange things that you can do
with Python's reflection capabilities.
"""
from __future__ import absolute_import

import sys

from .compat import PY3


class _NoModuleFound(Exception):
    """
    No module was found because none exists.
    """



class InvalidName(ValueError):
    """
    The given name is not a dot-separated list of Python objects.
    """



class ModuleNotFound(InvalidName):
    """
    The module associated with the given name doesn't exist and it can't be
    imported.
    """



class ObjectNotFound(InvalidName):
    """
    The object associated with the given name doesn't exist and it can't be
    imported.
    """



if PY3:
    def reraise(exception, traceback):
        raise exception.with_traceback(traceback)
else:
    exec("""def reraise(exception, traceback):
        raise exception.__class__, exception, traceback""")

reraise.__doc__ = """
Re-raise an exception, with an optional traceback, in a way that is compatible
with both Python 2 and Python 3.

Note that on Python 3, re-raised exceptions will be mutated, with their
C{__traceback__} attribute being set.

@param exception: The exception instance.
@param traceback: The traceback to use, or C{None} indicating a new traceback.
"""


def _importAndCheckStack(importName):
    """
    Import the given name as a module, then walk the stack to determine whether
    the failure was the module not existing, or some code in the module (for
    example a dependent import) failing.  This can be helpful to determine
    whether any actual application code was run.  For example, to distiguish
    administrative error (entering the wrong module name), from programmer
    error (writing buggy code in a module that fails to import).

    @param importName: The name of the module to import.
    @type importName: C{str}
    @raise Exception: if something bad happens.  This can be any type of
        exception, since nobody knows what loading some arbitrary code might
        do.
    @raise _NoModuleFound: if no module was found.
    """
    try:
        return __import__(importName)
    except ImportError:
        excType, excValue, excTraceback = sys.exc_info()
        while excTraceback:
            execName = excTraceback.tb_frame.f_globals["__name__"]
            # in Python 2 execName is None when an ImportError is encountered,
            # where in Python 3 execName is equal to the importName.
            if execName is None or execName == importName:
                reraise(excValue, excTraceback)
            excTraceback = excTraceback.tb_next
        raise _NoModuleFound()



def namedAny(name):
    """
    Retrieve a Python object by its fully qualified name from the global Python
    module namespace.  The first part of the name, that describes a module,
    will be discovered and imported.  Each subsequent part of the name is
    treated as the name of an attribute of the object specified by all of the
    name which came before it.  For example, the fully-qualified name of this
    object is 'twisted.python.reflect.namedAny'.

    @type name: L{str}
    @param name: The name of the object to return.

    @raise InvalidName: If the name is an empty string, starts or ends with
        a '.', or is otherwise syntactically incorrect.

    @raise ModuleNotFound: If the name is syntactically correct but the
        module it specifies cannot be imported because it does not appear to
        exist.

    @raise ObjectNotFound: If the name is syntactically correct, includes at
        least one '.', but the module it specifies cannot be imported because
        it does not appear to exist.

    @raise AttributeError: If an attribute of an object along the way cannot be
        accessed, or a module along the way is not found.

    @return: the Python object identified by 'name'.
    """
    if not name:
        raise InvalidName('Empty module name')

    names = name.split('.')

    # if the name starts or ends with a '.' or contains '..', the __import__
    # will raise an 'Empty module name' error. This will provide a better error
    # message.
    if '' in names:
        raise InvalidName(
            "name must be a string giving a '.'-separated list of Python "
            "identifiers, not %r" % (name,))

    topLevelPackage = None
    moduleNames = names[:]
    while not topLevelPackage:
        if moduleNames:
            trialname = '.'.join(moduleNames)
            try:
                topLevelPackage = _importAndCheckStack(trialname)
            except _NoModuleFound:
                moduleNames.pop()
        else:
            if len(names) == 1:
                raise ModuleNotFound("No module named %r" % (name,))
            else:
                raise ObjectNotFound('%r does not name an object' % (name,))

    obj = topLevelPackage
    for n in names[1:]:
        obj = getattr(obj, n)

    return obj



import json
import os.path
import shutil
import tempfile

from egginst._compat import assertCountEqual
from egginst.vendor.six.moves import unittest

from enstaller.compat import path_to_uri
from enstaller.config import Configuration
from enstaller.legacy_stores import parse_index, repository_factory

from enstaller.tests.common import (SIMPLE_INDEX,
                                    dummy_repository_package_factory,
                                    mock_index,
                                    mocked_session_factory)


def _index_provider(store_location):
    entries = [
        dummy_repository_package_factory("numpy", "1.8.0", 1,
                                         store_location=store_location),
        dummy_repository_package_factory("scipy", "0.14.0", 1,
                                         store_location=store_location)
    ]
    return json.dumps(dict((entry.key, entry.s3index_data) for entry in entries))


class TestLegacyStores(unittest.TestCase):
    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_simple_webservice(self):
        # Given
        store_location = ""
        body = _index_provider(store_location)

        # When
        packages = list(parse_index(json.loads(body), store_location))

        # Then
        self.assertTrue(len(packages) > 0)
        assertCountEqual(self, [p.name for p in packages], ["numpy", "scipy"])
        assertCountEqual(self, [p.full_version for p in packages], ["1.8.0-1",
                                                                    "0.14.0-1"])

    def test_simple_no_webservice_file(self):
        # Given
        fake_index = {
            "zope.testing-3.8.3-1.egg": {
                "available": True,
                "build": 1,
                "md5": "6041fd75b7fe9187ccef0d40332c6c16",
                "mtime": 1262725254.0,
                "name": "zope.testing",
                "product": "commercial",
                "python": "2.6",
                "size": 514439,
                "type": "egg",
                "version": "3.8.3",
            }
        }
        index_path = os.path.join(self.tempdir, "index.json")
        with open(index_path, "w") as fp:
            fp.write(json.dumps(fake_index))

        # When
        session = mocked_session_factory(self.tempdir)
        resp = session.fetch(path_to_uri(index_path))
        packages = list(parse_index(resp.json(), "", python_version="2.6"))

        # Then
        self.assertEqual(len(packages), 1)
        self.assertEqual(packages[0].key, "zope.testing-3.8.3-1.egg")

    def test_parse_index_python_version(self):
        # Given
        index = {
            "zope.testing-3.8.3-1.egg": {
                "available": True,
                "build": 1,
                "md5": "6041fd75b7fe9187ccef0d40332c6c16",
                "mtime": 1262725254.0,
                "name": "zope.testing",
                "product": "commercial",
                "python": "2.6",
                "size": 514439,
                "type": "egg",
                "version": "3.8.3",
            }
        }

        # When
        packages = list(parse_index(index, "", python_version="2.6"))

        # Then
        self.assertEqual(len(packages), 1)

        # When
        packages = list(parse_index(index, "", python_version="2.4"))

        # Then
        self.assertEqual(len(packages), 0)

        # When
        packages = list(parse_index(index, "", "*"))

        # Then
        self.assertEqual(len(packages), 1)

    @mock_index(SIMPLE_INDEX)
    def test_repository_factory(self):
        # Given
        config = Configuration()
        session = mocked_session_factory(self.tempdir)

        # When
        repository = repository_factory(session, config.indices)

        # Then
        self.assertEqual(len(list(repository.iter_packages())), 1)
        self.assertEqual(len(list(repository.find_packages("nose", "1.3.4-1"))), 1)



import sys

from egginst.vendor.six.moves import unittest

from enstaller.errors import InvalidConfiguration
from enstaller.proxy_info import ProxyInfo


class TestProxyInfoFromString(unittest.TestCase):
    def test_without_host(self):
        # Given
        s = ""

        # When/Then
        with self.assertRaises(InvalidConfiguration):
            ProxyInfo.from_string(s)

        # Given
        s = ":3128"

        # When/Then
        with self.assertRaises(InvalidConfiguration):
            ProxyInfo.from_string(s)

    @unittest.skipIf(sys.version_info < (2, 7),
                     "Bug in stdlib on 2.6")
    def test_without_scheme(self):
        # Given
        s = "acme.com:3129"

        # When
        info = ProxyInfo.from_string(s)

        # Then
        self.assertEqual(info.scheme, "http")
        self.assertEqual(info.host, "acme.com")
        self.assertEqual(info.user, "")
        self.assertEqual(info.password, "")
        self.assertEqual(info.port, 3129)

    def test_simple(self):
        # Given
        s = "http://acme.com"

        # When
        info = ProxyInfo.from_string(s)

        # Then
        self.assertEqual(info.scheme, "http")
        self.assertEqual(info.user, "")
        self.assertEqual(info.password, "")
        self.assertEqual(info.port, 3128)

    def test_simple_port(self):
        # Given
        s = "http://acme.com:3129"

        # When
        info = ProxyInfo.from_string(s)

        # Then
        self.assertEqual(info.scheme, "http")
        self.assertEqual(info.user, "")
        self.assertEqual(info.password, "")
        self.assertEqual(info.port, 3129)

    def test_simple_user(self):
        # Given
        s = "http://john:doe@acme.com"

        # When
        info = ProxyInfo.from_string(s)

        # Then
        self.assertEqual(info.scheme, "http")
        self.assertEqual(info.user, "john")
        self.assertEqual(info.password, "doe")
        self.assertEqual(info.port, 3128)

    def test_simple_user_wo_password(self):
        # Given
        s = "http://john@acme.com"

        # When
        info = ProxyInfo.from_string(s)

        # Then
        self.assertEqual(info.scheme, "http")
        self.assertEqual(info.user, "john")
        self.assertEqual(info.password, "")
        self.assertEqual(info.port, 3128)

    def test_scheme(self):
        # Given
        s = "https://john@acme.com"

        # When
        info = ProxyInfo.from_string(s)

        # Then
        self.assertEqual(info.scheme, "https")
        self.assertEqual(info.user, "john")
        self.assertEqual(info.password, "")
        self.assertEqual(info.port, 3128)


class TestProxyInfo(unittest.TestCase):
    def test_no_user(self):
        with self.assertRaises(InvalidConfiguration):
            ProxyInfo("acme.com", password="yoyo")

    def test_str_simple(self):
        # Given
        proxy_info = ProxyInfo.from_string("http://acme.com:3129")

        # When
        s = str(proxy_info)

        # Then
        self.assertEqual(s, "http://acme.com:3129")

    def test_str_full(self):
        # Given
        proxy_info = ProxyInfo.from_string("http://john:doe@acme.com:3129")

        # When
        s = str(proxy_info)

        # Then
        self.assertEqual(s, "http://john:doe@acme.com:3129")



"""
Naive implementation of freeze-like feature
"""
from enstaller.repository import Repository


def get_freeze_list(prefixes):
    """
    Compute the list of eggs installed in the given prefixes.

    Returns
    -------
    names: seq
        List of installed eggs, as full names (e.g. 'numpy-1.8.0-1')
    """
    full_names = [
        "{0} {1}".format(package.name, package.full_version)
        for package in Repository._from_prefixes(prefixes).iter_packages()
    ]
    return sorted(full_names)



import sys

if sys.version_info[0] >= 3:
    PY3 = True
    PY2 = False
else:
    PY3 = False
    PY2 = True

if PY3:
    def is_unicode(s):
        return isinstance(s, str)

    unicode = str
else:
    def is_unicode(s):
        return isinstance(s, unicode)

    unicode = unicode



from __future__ import print_function

import argparse
import sys

import enstaller.plat

from enstaller.cli.utils import repository_factory
from enstaller.config import Configuration
from enstaller.session import Session
from enstaller.solver import Requirement
from enstaller.solver.resolve import Resolve


def query_platform(session, indices, requirement, platform):
    repository = repository_factory(session, indices)

    requirement = Requirement(requirement)
    resolve = Resolve(repository)

    def print_level(parent, level=0):
        level += 4
        for r in resolve._dependencies_from_egg(parent):
            print("{0}{1}".format(level * " ", r))
            egg = resolve._latest_egg(r)
            if egg is None:
                msg = "Error: Could not find egg for requirement {0!r}"
                print(msg.format(r))
                sys.exit(-1)
            print_level(egg, level)

    root = resolve._latest_egg(requirement)
    if root is None:
        print("No egg found for requirement {0}".format(requirement))
    else:
        print("Resolving dependencies for {0}: {1}".format(requirement, root))
        print_level(root)


def main(argv=None):
    argv = argv or sys.argv[1:]

    plat = enstaller.plat.custom_plat

    p = argparse.ArgumentParser()
    p.add_argument("requirement",
                   help="Requirement string (e.g. 'mayavi')")
    p.add_argument("--platform",
                   help="Platform to consider (default: %(default)s). 'all' works as well",
                   default=plat)
    p.add_argument("--auth",
                   help="Authentication (default: enpkg credentials)")

    namespace = p.parse_args(argv)

    config = Configuration._from_legacy_locations()
    config._platform = namespace.platform

    if namespace.auth is None:
        auth = config.auth
    else:
        auth = tuple(namespace.auth.split(":"))

    session = Session.from_configuration(config)
    with session:
        session.authenticate(auth)

        if namespace.platform == "all":
            platforms = ["rh5-32", "rh5-64", "osx-32", "osx-64", "win-32", "win-64"]
            for platform in platforms:
                query_platform(session, config.indices, namespace.requirement,
                               platform)
        else:
            query_platform(session, config.indices, namespace.requirement,
                           namespace.platform)


if __name__ == "__main__":  # pragma: nocover
    main()



# Author: Ilan Schnell <ischnell@enthought.com>
"""\
egginst is a simple tool for installing and uninstalling eggs. Example:

    egginst nose-1.3.0-1.egg

This tool is simple and does not care if the eggs it installs are for the
correct platform, its dependencies installed, etc... You should generally use
enpkg or Canopy's package manager instead to deal with dependencies correctly.
"""
from __future__ import absolute_import, print_function

import argparse
import json
import logging
import os
import posixpath
import re
import shutil
import subprocess
import sys
import warnings

from os.path import abspath, basename, dirname, join, isdir, isfile, normpath, sep

try:
    import appinst
except ImportError:  # pragma: no cover
    appinst = None

from . import eggmeta
from . import object_code
from . import scripts

from .links import create_link
from .progress import console_progress_manager_factory
from .utils import (on_win, bin_dir_name, rel_site_packages, ensure_dir,
                    rm_empty_dir, rm_rf, get_executable, is_zipinfo_dir,
                    zip_has_arcname)
from ._zipfile import ZipFile

from .vendor.six import StringIO
from .vendor.six.moves import configparser

EGG_INFO = "EGG-INFO"
BOOTSTRAP_ARCNAME = EGG_INFO + "/spec/__bootstrap__.py"

R_EGG_INFO = re.compile("^{0}".format(EGG_INFO))
R_EGG_INFO_BLACK_LIST = re.compile(
    "^{0}/(usr|spec|PKG-INFO.bak|prefix|.gitignore|"
    "inst|post_egginst.py|pre_egguninst.py)".format(EGG_INFO))
R_LEGACY_EGG_INFO = re.compile("(^.+.egg-info)")

PY_PAT = re.compile(r'^(.+)\.py(c|o)?$')
SO_PAT = re.compile(r'^lib.+\.so')
PY_OBJ = '.pyd' if on_win else '.so'

logger = logging.getLogger(__name__)


def name_version_fn(fn):
    """
    Given the filename of a package, returns a tuple(name, version).
    """
    if fn.endswith('.egg'):
        fn = fn[:-4]
    if '-' in fn:
        return tuple(fn.split('-', 1))
    else:
        return fn, ''


def is_in_legacy_egg_info(f, is_custom_egg):
    """
    Filter out files in legacy egg-info directory, i.e. for eggs as follows:

        package/__init__.py
        EGG-INFO/...
        package.egg-info/...

    filter out the content of package.egg-info. Only a few eggs were produced
    as above. This version of enstaller will simply copy the required content
    from EGG-INFO where setuptools expects it.
    """
    if is_custom_egg:
        if R_LEGACY_EGG_INFO.search(f):
            return True
        else:
            return False
    else:
        return False


def should_copy_in_egg_info(f, is_custom_egg):
    """
    Return True if the given archive name needs to be copied in to
    $site-packages/$package_egg_info
    """
    if R_EGG_INFO.search(f):
        if is_custom_egg:
            if R_EGG_INFO_BLACK_LIST.search(f):
                return False
            else:
                return True
        else:
            return True
    else:
        return False


def has_legacy_egg_info_format(arcnames, is_custom_egg):
    if is_custom_egg:
        for name in arcnames:
            if R_LEGACY_EGG_INFO.search(name):
                return True
        return False
    else:
        return False


def should_mark_executable(arcname, fn):
    if os.path.islink(fn):
        return False
    if (arcname.startswith(('EGG-INFO/usr/bin/', 'EGG-INFO/scripts/')) or
            fn.endswith(('.dylib', '.pyd', '.so')) or
            (arcname.startswith('EGG-INFO/usr/lib/') and
             SO_PAT.match(fn))):
        return True
    else:
        return False


def should_skip(zp, arcname):
    m = PY_PAT.match(arcname)
    if m and zip_has_arcname(zp, m.group(1) + PY_OBJ):
        # .py, .pyc, .pyo next to .so are not written
        return True
    else:
        return False


def setuptools_egg_info_dir(path):
    """
    Return the .egg-info directory name as created/expected by setuptools
    """
    filename = basename(path)
    name, version = name_version_fn(filename)
    return "{0}-{1}.egg-info".format(name, version)


def install_app(meta_dir, prefix):
    return _install_app_impl(meta_dir, prefix, remove=False)


def remove_app(meta_dir, prefix):
    return _install_app_impl(meta_dir, prefix, remove=True)


def _install_app_impl(meta_dir, prefix, remove=False):
    if appinst is None:
        return

    path = join(meta_dir, eggmeta.APPINST_PATH)
    if not isfile(path):
        return

    if remove:
        handler = appinst.uninstall_from_dat
        warning = 'uninstalling application item'
    else:
        handler = appinst.install_from_dat
        warning = 'installing application item'

    try:
        try:
            handler(path, prefix)
        except TypeError:
            # Old appinst (<= 2.1.1) did not handle the prefix argument (2d
            # arg)
            handler(path)
    except Exception as e:
        logger.warn("Warning ({0}):\n{1!r}".format(warning, e))


def _run_script(meta_dir, fn, prefix):
    path = join(meta_dir, fn)
    if not isfile(path):
        return
    subprocess.call([scripts.executable, '-E', path, '--prefix', prefix],
                    cwd=dirname(path))


class _EggInstRemove(object):

    def __init__(self, path, prefix=sys.prefix, noapp=False):
        self.path = path
        self.fn = basename(path)
        name, version = name_version_fn(self.fn)
        self.cname = name.lower()
        self.prefix = abspath(prefix)
        self.noapp = noapp

        self.egginfo_dir = join(self.prefix, 'EGG-INFO')
        self.meta_dir = join(self.egginfo_dir, self.cname)

        self._files = None
        self._installed_size = None

    @property
    def is_installed(self):
        return isdir(self.meta_dir)

    @property
    def files(self):
        if self._files is None:
            self._read_uninstall_metadata()
        return self._files

    @property
    def installed_size(self):
        if self._installed_size is None:
            self._read_uninstall_metadata()
        return self._installed_size

    def _read_uninstall_metadata(self):
        d = read_meta(self.meta_dir)

        self._files = [join(self.prefix, f) for f in d['files']]
        self._installed_size = d['installed_size']

    def _rm_dirs(self, files):
        dir_paths = set()
        len_prefix = len(self.prefix)
        for path in set(dirname(p) for p in files):
            while len(path) > len_prefix:
                dir_paths.add(path)
                path = dirname(path)

        for path in sorted(dir_paths, key=len, reverse=True):
            if not path.rstrip(sep).endswith('site-packages'):
                rm_empty_dir(path)

    def remove_iterator(self):
        """
        Create an iterator that will remove every installed file.

        Example::

            from egginst.console import ProgressManager

            progress = ProgressManager(...)
            egginst = EggInst(...)

            with progress:
                for i, filename in self.remove_iterator():
                    print("removing file {0}".format(filename))
                    progress(step=i)
        """
        if not self.is_installed:
            logger.error("Error: Can't find meta data for: {0!r}".
                         format(self.cname))
            return

        if not self.noapp:
            remove_app(self.meta_dir, self.prefix)
        _run_script(self.meta_dir, 'pre_egguninst.py', self.prefix)

        for n, p in enumerate(self.files):
            n += 1

            rm_rf(p)
            if p.endswith('.py'):
                rm_rf(p + 'c')
                rm_rf(p + 'o')

            yield p

        self._rm_dirs(self.files)
        rm_rf(self.meta_dir)
        rm_empty_dir(self.egginfo_dir)

    def remove(self):
        for filename in self.remove_iterator():
            pass


class EggInst(object):

    def __init__(self, path, prefix=sys.prefix, hook=False, pkgs_dir=None,
                 noapp=False):
        self.path = path
        self.fn = basename(path)
        name, version = name_version_fn(self.fn)
        self.cname = name.lower()
        self.prefix = abspath(prefix)
        self.noapp = noapp

        self.bin_dir = join(self.prefix, bin_dir_name)

        if self.prefix != abspath(sys.prefix):
            scripts.executable = get_executable(self.prefix)

        self.site_packages = join(self.prefix, rel_site_packages)
        self.pyloc = self.site_packages
        self.egginfo_dir = join(self.prefix, 'EGG-INFO')
        self.meta_dir = join(self.egginfo_dir, self.cname)

        self.meta_json = join(self.meta_dir, 'egginst.json')
        self.files = []

        self._egginst_remover = _EggInstRemove(path, prefix, noapp)
        self._installed_size = None
        self._files_to_install = None

    @property
    def installed_size(self):
        """
        Return the size (bytes) of the extracted egg.
        """
        if self._installed_size is None:
            with ZipFile(self.path) as zp:
                self._installed_size = sum(zp.getinfo(name).file_size for name
                                           in zp.namelist())
        return self._installed_size

    def iter_files_to_install(self):
        return self._lines_from_arcname('EGG-INFO/inst/files_to_install.txt')

    def iter_targets(self):
        return self._lines_from_arcname('EGG-INFO/inst/targets.dat')

    def _should_create_info(self):
        for arcname in ('EGG-INFO/spec/depend', 'EGG-INFO/info.json'):
            if zip_has_arcname(self.z, arcname):
                return True
        return False

    def pre_extract(self):
        if not isdir(self.meta_dir):
            os.makedirs(self.meta_dir)

    def post_extract(self, extra_info=None):
        with ZipFile(self.path) as zp:
            self.z = zp

            if on_win:
                scripts.create_proxies(self)
            else:
                # XXX: we ignore placeholder hack for enstaller, to avoid error
                # messages related to tests data when updating enstaller
                # (enstaller test data contain some osx/linux binaries)
                if self.cname != "enstaller":
                    object_code.apply_placeholder_hack(self.files,
                                                       list(self.iter_targets()),
                                                       self.prefix)

                self._create_links()

            self._entry_points()
            if self._should_create_info():
                eggmeta.create_info(self, extra_info)

        scripts.fix_scripts(self)

        if not self.noapp:
            install_app(self.meta_dir, self.prefix)

        self._write_meta()

        _run_script(self.meta_dir, 'post_egginst.py', self.prefix)

    def install(self, extra_info=None):
        for currently_extracted_size in self.install_iterator():
            pass

    def _create_links(self):
        """
        Given the content of the EGG-INFO/inst/files_to_install.txt file,
        create/remove the links listed therein.
        """
        for line in self.iter_files_to_install():
            arcname, link = line.split()
            if link == 'False':
                continue
            self.files.append(create_link(arcname, link, self.prefix))

    def _entry_points(self):
        lines = list(self._lines_from_arcname('EGG-INFO/entry_points.txt',
                                              ignore_empty=False))
        if lines == []:
            return
        conf = configparser.ConfigParser()
        data = u'\n'.join(lines) + '\n'
        # XXX: hack to workaround 2.6-specific bug with ConfigParser and
        # unicode.
        if sys.version_info < (2, 7, 3):
            conf.readfp(StringIO(data.encode("utf8")))
        else:
            conf.readfp(StringIO(data))
        if ('console_scripts' in conf.sections() or
                'gui_scripts' in conf.sections()):
            logger.debug('creating scripts')
            scripts.create(self, conf)

    def _rel_prefix(self, path):
        return abspath(path).replace(self.prefix, '.').replace('\\', '/')

    def _write_meta(self):
        d = dict(
            egg_name=self.fn,
            prefix=self.prefix,
            installed_size=self.installed_size,
            files=[self._rel_prefix(p)
                   if abspath(p).startswith(self.prefix) else p
                   for p in self.files + [self.meta_json]]
        )
        with open(self.meta_json, 'w') as f:
            json.dump(d, f, indent=2, sort_keys=True)

    def _lines_from_arcname(self, arcname, ignore_empty=True):
        if zip_has_arcname(self.z, arcname):
            for line in self.z.read(arcname).decode("utf8").splitlines():
                line = line.strip()
                if ignore_empty and line == '':
                    continue
                if line.startswith('#'):
                    continue
                yield line

    def install_iterator(self, extra_info=None):
        """
        Create an iterator that will iterate over each archive to be extracted.

        Example::

            from egginst.console import ProgressManager

            progress = ProgressManager(...)
            egginst = EggInst(...)

            with progress:
                for n in self.install_iterator():
                    progress(step=n)
        """
        self.pre_extract()

        with ZipFile(self.path) as zp:
            self.z = zp

            arcnames = self.z.namelist()
            is_custom_egg = eggmeta.is_custom_egg(self.path)

            use_legacy_egg_info_format = has_legacy_egg_info_format(arcnames,
                                                                    is_custom_egg)

            for arcname in arcnames:
                if use_legacy_egg_info_format:
                    n = self._extract_egg_with_legacy_egg_info(arcname,
                                                               is_custom_egg)
                else:
                    n = self._extract(arcname, is_custom_egg)
                yield n

        self.post_extract(extra_info)

    def _extract_egg_with_legacy_egg_info(self, name, is_custom_egg):
        zip_info = self.z.getinfo(name)

        if is_in_legacy_egg_info(name, is_custom_egg):
            self._write_legacy_egg_info_metadata(zip_info)
        else:
            self._write_arcname(name)

        return zip_info.file_size

    def _extract(self, name, is_custom_egg):
        zip_info = self.z.getinfo(name)

        self._write_arcname(name)
        if should_copy_in_egg_info(name, is_custom_egg):
            self._write_standard_egg_info_metadata(zip_info)

        return zip_info.file_size

    def _write_legacy_egg_info_metadata(self, zip_info):
        if is_zipinfo_dir(zip_info):
            return

        name = zip_info.filename
        m = R_LEGACY_EGG_INFO.search(name)
        if m:
            legacy_egg_info_dir = m.group(1)
            from_egg_info = posixpath.relpath(name, legacy_egg_info_dir)

            dest = join(self.pyloc, setuptools_egg_info_dir(self.path),
                        from_egg_info)
            self._write_egg_info_arcname(name, dest)
        else:
            msg = ("BUG: Unexpected name for legacy egg info in {0}: {1}".
                   format(self.fn, name))
            raise ValueError(msg)

    def _write_standard_egg_info_metadata(self, zip_info):
        if is_zipinfo_dir(zip_info):
            return

        name = zip_info.filename
        from_egg_info = posixpath.relpath(name, EGG_INFO)
        dest = posixpath.join(self.pyloc, setuptools_egg_info_dir(self.path),
                              from_egg_info)

        self._write_egg_info_arcname(name, dest)

    def _write_egg_info_arcname(self, name, dest):
        ensure_dir(dest)
        source = self.z.open(name)
        try:
            with open(dest, "wb") as target:
                shutil.copyfileobj(source, target)
                self.files.append(dest)
        finally:
            source.close()

    def _get_dst(self, arcname):
        def _transform_path(arcname, egg_prefix, dest_prefix):
            return abspath(join(dest_prefix, arcname[len(egg_prefix):]))

        if on_win:
            scheme = [
                ("EGG-INFO/prefix/", self.prefix),
                ("EGG-INFO/scripts/", self.bin_dir),
                ("EGG-INFO/", self.meta_dir),
            ]
        else:
            scheme = [
                ("EGG-INFO/prefix/", self.prefix),
                ("EGG-INFO/usr/", self.prefix),
                ("EGG-INFO/scripts/", self.bin_dir),
                ("EGG-INFO/", self.meta_dir),
            ]

        for prefix, dest in scheme:
            if arcname.startswith(prefix):
                return _transform_path(arcname, prefix, dest)
        return _transform_path(arcname, "", self.pyloc)

    def _write_arcname(self, arcname):
        if arcname.endswith('/') or arcname.startswith('.unused'):
            return

        if should_skip(self.z, arcname):
            return

        path = self._get_dst(arcname)
        destination = os.path.relpath(path, self.prefix)

        self.z.extract_to(arcname, destination, self.prefix)
        self.files.append(path)

        if should_mark_executable(arcname, path):
            os.chmod(path, 0o755)

    def remove(self):
        return self._egginst_remover.remove()

    def remove_iterator(self):
        return self._egginst_remover.remove_iterator()


def read_meta(meta_dir):
    meta_json = join(meta_dir, 'egginst.json')
    if isfile(meta_json):
        with open(meta_json) as fp:
            return json.load(fp)
    return None


def get_installed(prefix=sys.prefix):
    """
    Generator returns a sorted list of all installed packages.
    Each element is the filename of the egg which was used to install the
    package.
    """
    egg_info_dir = join(prefix, 'EGG-INFO')
    if not isdir(egg_info_dir):
        return
    pat = re.compile(r'([a-z0-9_.]+)$')
    for fn in sorted(os.listdir(egg_info_dir)):
        if not pat.match(fn):
            continue
        d = read_meta(join(egg_info_dir, fn))
        if d is None:
            continue
        yield d['egg_name']


def print_installed(prefix=sys.prefix):
    fmt = '%-20s %s'
    print(fmt % ('Project name', 'Version'))
    print(40 * '=')
    for fn in get_installed(prefix):
        print(fmt % name_version_fn(fn))


def install_egg_cli(path, prefix, noapp=False, extra_info=None):
    """
    Simple wrapper to install an egg using default egginst progress bar.
    """
    installer = EggInst(path, prefix, False, None, noapp)

    progress = console_progress_manager_factory("installing egg", installer.fn,
                                                size=installer.installed_size)
    with progress:
        for currently_extracted_size in installer.install_iterator(extra_info):
            progress.update(currently_extracted_size)


def remove_egg_cli(path, prefix, noapp=False):
    """
    Simple wrapper to remove an egg using default egginst progress bar.
    """
    installer = EggInst(path, prefix, False, None, noapp=noapp)
    remover = installer._egginst_remover
    if not remover.is_installed:
        logger.error("Error: can't find meta data for: %r", remover.cname)
        return
    progress = console_progress_manager_factory("removing egg", installer.fn,
                                                remover.installed_size,
                                                len(remover.files))
    with progress:
        for filename in remover.remove_iterator():
            progress.update(1)


def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]  # pragma: no cover

    p = argparse.ArgumentParser(usage="usage: %(prog)s [options] [EGGS ...]",
                                description=__doc__,
                                formatter_class=argparse.RawTextHelpFormatter)

    p.add_argument("requirements", help="Requirements to install", nargs='*')

    p.add_argument('-l', "--list",
                   action="store_true",
                   help="list all installed packages")

    p.add_argument("--noapp",
                   action="store_true",
                   help="don't install/remove application menu items")

    p.add_argument("--prefix",
                   action="store",
                   default=sys.prefix,
                   help="install prefix",
                   metavar='PATH')

    p.add_argument("--pkgs-dir",
                   action="store",
                   help="Do nothing, kept for backward compatibility.",
                   metavar='PATH')

    p.add_argument('-r', "--remove",
                   action="store_true",
                   help="remove package(s), requires the egg or project name(s)")

    p.add_argument('-v', "--verbose", action="store_true")
    p.add_argument('--version', action="store_true")

    ns = p.parse_args(argv)
    if ns.version:
        # Local import to avoid circular imports between enstaller and egginst
        from enstaller import __version__
        print("enstaller version:", __version__)
        return

    prefix = normpath(abspath(ns.prefix))
    if prefix != normpath(sys.prefix):
        warnings.warn("Using the --prefix option is potentially dangerous. "
                      "You should use enpkg installed in {0} instead.".
                      format(ns.prefix))

    if ns.list:
        print_installed(prefix)
        return

    if ns.verbose:
        logging.basicConfig(level=logging.INFO, format="%(message)s")
    else:
        logging.basicConfig(level=logging.WARN, format="%(message)s")

    for path in ns.requirements:
        if ns.remove:
            remove_egg_cli(path, prefix, ns.noapp)
        else:
            install_egg_cli(path, prefix, ns.noapp)


if __name__ == '__main__':  # pragma: no cover
    main()



import hashlib
import os.path
import sys

import mock

from egginst import exe_data

from egginst.main import EggInst
from egginst.scripts import create, create_proxies, fix_script, get_executable
from egginst.utils import compute_md5
from egginst.vendor.six import PY2, StringIO
from egginst.vendor.six.moves import configparser
from egginst._zipfile import ZipFile

from egginst.vendor.six.moves import unittest

from .common import mkdtemp

DUMMY_EGG_WITH_PROXY = os.path.join(os.path.dirname(__file__), "data", "dummy_with_proxy-1.3.40-3.egg")
DUMMY_EGG_WITH_PROXY_SCRIPTS = os.path.join(os.path.dirname(__file__), "data", "dummy_with_proxy_scripts-1.0.0-1.egg")


class TestScripts(unittest.TestCase):
    def test_get_executable(self):
        # FIXME: EggInst init overwrite egginst.scripts.executable. Need to
        # mock this until we remove that insanity
        with mock.patch("egginst.scripts.executable", sys.executable):
            executable = get_executable()
            self.assertEqual(executable, sys.executable)

            executable = get_executable(with_quotes=True)
            self.assertEqual(executable, "\"{0}\"".format(sys.executable))

        with mock.patch("egginst.scripts.on_win", "win32"):
            with mock.patch("egginst.scripts.executable", "python.exe"):
                executable = get_executable()
                self.assertEqual(executable, "python.exe")

            with mock.patch("egginst.scripts.executable", "pythonw.exe"):
                executable = get_executable()
                self.assertEqual(executable, "python.exe")

                executable = get_executable(pythonw=True)
                self.assertEqual(executable, "pythonw.exe")


class TestFixScript(unittest.TestCase):
    def test_egginst_script_untouched(self):
        """
        Ensure we don't touch a script which has already been written by
        egginst.
        """
        simple_script = """\
#!/home/davidc/src/enthought/enstaller/.env/bin/python
# This script was created by egginst when installing:
#
#   enstaller-4.6.3.dev1-py2.7.egg
#
if __name__ == '__main__':
    import sys
    from enstaller.patch import main

    sys.exit(main())
"""

        with mkdtemp() as d:
            path = os.path.join(d, "script")
            with open(path, "wt") as fp:
                fp.write(simple_script)

            fix_script(path)

            with open(path, "rt") as fp:
                self.assertEqual(fp.read(), simple_script)

    @mock.patch("egginst.scripts.executable", sys.executable)
    def test_setuptools_script_fixed(self):
        """
        Ensure a script generated by setuptools is fixed.
        """
        setuptools_script = """\
#!/dummy_path/.env/bin/python
# EASY-INSTALL-ENTRY-SCRIPT: 'enstaller==4.6.3.dev1','console_scripts','enpkg'
__requires__ = 'enstaller==4.6.3.dev1'
import sys
from pkg_resources import load_entry_point

if __name__ == '__main__':
    sys.exit(
        load_entry_point('enstaller==4.6.3.dev1', 'console_scripts', 'enpkg')()
    )
"""
        if sys.platform == "win32":
            executable = '"' + sys.executable + '"'
        else:
            executable = sys.executable
        r_egginst_script = """\
#!{executable}
# EASY-INSTALL-ENTRY-SCRIPT: 'enstaller==4.6.3.dev1','console_scripts','enpkg'
__requires__ = 'enstaller==4.6.3.dev1'
import sys
from pkg_resources import load_entry_point

if __name__ == '__main__':
    sys.exit(
        load_entry_point('enstaller==4.6.3.dev1', 'console_scripts', 'enpkg')()
    )
""".format(executable=executable)

        with mkdtemp() as d:
            path = os.path.join(d, "script")
            with open(path, "wt") as fp:
                fp.write(setuptools_script)

            fix_script(path)

            with open(path, "rt") as fp:
                self.assertMultiLineEqual(fp.read(), r_egginst_script)


class TestCreateScript(unittest.TestCase):
    @mock.patch("egginst.utils.on_win", False)
    def test_simple(self):
        if sys.platform == "win32":
            q = "\""
        else:
            q = ""
        r_cli_entry_point = """\
#!{q}{executable}{q}
# This script was created by egginst when installing:
#
#   dummy.egg
#
if __name__ == '__main__':
    import sys
    from dummy import main_cli

    sys.exit(main_cli())
""".format(executable=sys.executable, q=q)

        entry_points = """\
[console_scripts]
dummy = dummy:main_cli

[gui_scripts]
dummy-gui = dummy:main_gui
"""
        s = StringIO(entry_points)
        config = configparser.ConfigParser()
        config.readfp(s)

        with mkdtemp() as d:
            egginst = EggInst("dummy.egg", d)
            create(egginst, config)

            if sys.platform == "win32":
                entry_point = os.path.join(egginst.bin_dir, "dummy-script.py")
            else:
                entry_point = os.path.join(egginst.bin_dir, "dummy")
            self.assertTrue(os.path.exists(entry_point))

            with open(entry_point, "rt") as fp:
                cli_entry_point = fp.read()
                self.assertMultiLineEqual(cli_entry_point, r_cli_entry_point)

    @mock.patch("egginst.scripts.on_win", True)
    @mock.patch("egginst.main.bin_dir_name", "Scripts")
    def test_simple_windows(self):
        python_executable = "C:\\Python27\\python.exe"
        pythonw_executable = "C:\\Python27\\pythonw.exe"

        r_cli_entry_point = """\
#!"{executable}"
# This script was created by egginst when installing:
#
#   dummy.egg
#
if __name__ == '__main__':
    import sys
    from dummy import main_cli

    sys.exit(main_cli())
""".format(executable=python_executable)

        r_gui_entry_point = """\
#!"{executable}"
# This script was created by egginst when installing:
#
#   dummy.egg
#
if __name__ == '__main__':
    import sys
    from dummy import main_gui

    sys.exit(main_gui())
""".format(executable=pythonw_executable)

        entry_points = """\
[console_scripts]
dummy = dummy:main_cli

[gui_scripts]
dummy-gui = dummy:main_gui
"""
        s = StringIO(entry_points)
        config = configparser.ConfigParser()
        config.readfp(s)

        with mock.patch("sys.executable", python_executable):
            with mkdtemp() as d:
                egginst = EggInst("dummy.egg", d)
                create(egginst, config)

                cli_entry_point_path = os.path.join(egginst.bin_dir, "dummy-script.py")
                gui_entry_point_path = os.path.join(egginst.bin_dir, "dummy-gui-script.pyw")
                entry_points = [
                    os.path.join(egginst.bin_dir, "dummy.exe"),
                    os.path.join(egginst.bin_dir, "dummy-gui.exe"),
                    cli_entry_point_path, gui_entry_point_path,
                ]
                for entry_point in entry_points:
                    self.assertTrue(os.path.exists(entry_point))

                with open(cli_entry_point_path, "rt") as fp:
                    cli_entry_point = fp.read()
                    self.assertMultiLineEqual(cli_entry_point, r_cli_entry_point)

                with open(gui_entry_point_path, "rt") as fp:
                    gui_entry_point = fp.read()
                    self.assertMultiLineEqual(gui_entry_point, r_gui_entry_point)

                self.assertEqual(compute_md5(os.path.join(egginst.bin_dir, "dummy.exe")),
                                 hashlib.md5(exe_data.cli).hexdigest())
                self.assertEqual(compute_md5(os.path.join(egginst.bin_dir, "dummy-gui.exe")),
                                 hashlib.md5(exe_data.gui).hexdigest())


class TestProxy(unittest.TestCase):
    @mock.patch("sys.platform", "win32")
    @mock.patch("egginst.main.bin_dir_name", "Scripts")
    @mock.patch("egginst.utils.on_win", True)
    def test_proxy(self):
        """
        Test we handle correctly entries of the form 'path PROXY'.
        """
        r_python_proxy_data_template = """\
#!"%(executable)s"
# This proxy was created by egginst from an egg with special instructions
#
import sys
import subprocess

src = %(src)r

sys.exit(subprocess.call([src] + sys.argv[1:]))
"""

        with mkdtemp() as prefix:
            with mock.patch("sys.executable", os.path.join(prefix, "python.exe")):
                proxy_path = os.path.join(prefix, "EGG-INFO", "dummy_with_proxy", "usr", "swig.exe")
                if PY2:
                    proxy_path = proxy_path.decode("utf8")
                r_python_proxy_data = r_python_proxy_data_template % \
                    {'executable': os.path.join(prefix, "python.exe"),
                     'src': proxy_path}

                egginst = EggInst(DUMMY_EGG_WITH_PROXY, prefix)
                with ZipFile(egginst.path) as zp:
                    egginst.z = zp
                    egginst.arcnames = zp.namelist()
                    create_proxies(egginst)

                    python_proxy = os.path.join(prefix, "Scripts", "swig-script.py")
                    coff_proxy = os.path.join(prefix, "Scripts", "swig.exe")

                    self.assertTrue(os.path.exists(python_proxy))
                    self.assertTrue(os.path.exists(coff_proxy))

                    self.assertTrue(compute_md5(coff_proxy),
                                    hashlib.md5(exe_data.cli).hexdigest())

                    with open(python_proxy, "rt") as fp:
                        python_proxy_data = fp.read()
                        self.assertMultiLineEqual(python_proxy_data,
                                                  r_python_proxy_data)

    @mock.patch("sys.platform", "win32")
    @mock.patch("egginst.main.bin_dir_name", "Scripts")
    @mock.patch("egginst.utils.on_win", True)
    def test_proxy_directory(self):
        """
        Test we handle correctly entries of the form 'path some_directory'.
        """
        with mkdtemp() as prefix:
            with mock.patch("sys.executable", os.path.join(prefix, "python.exe")):
                egginst = EggInst(DUMMY_EGG_WITH_PROXY_SCRIPTS, prefix)
                with ZipFile(egginst.path) as zp:
                    egginst.z = zp
                    egginst.arcnames = zp.namelist()
                    create_proxies(egginst)

                    proxied_files = [
                        os.path.join(prefix, "Scripts", "dummy.dll"),
                        os.path.join(prefix, "Scripts", "dummy.lib"),
                    ]
                    for proxied_file in proxied_files:
                        self.assertTrue(os.path.exists(proxied_file))



import posixpath
import sys
from os.path import abspath, dirname, normpath

import mock

from egginst.vendor.six.moves import unittest

from enstaller.utils import PY_VER
from enstaller.indexed_repo import Chain
import enstaller.indexed_repo.dist_naming as dist_naming
import enstaller.indexed_repo.requirement as requirement
from enstaller.indexed_repo.requirement import (Req, dist_as_req,
                                                add_Reqs_to_spec)

from enstaller.utils import path_to_uri

HERE_DIR = abspath(dirname(__file__))

class TestDistNaming(unittest.TestCase):

    def test_split_dist(self):
        for repo, fn in [
            ('http://www.example.com/repo/', 'foo.egg'),
            ('https://www.example.com/repo/', 'foo.egg'),
            ('file:///home/repo/', 'numpy-1.1.1-5.egg'),
            ('file://E:\\eggs\\', 'numpy-1.1.1-5.egg'),
            ('file://C:\\Desk and Top\\', 'with space.egg'),
            ]:
            dist = repo + fn
            self.assertEqual(dist_naming.split_dist(dist), (repo, fn))

        for dist in ['local:/foo.egg', '', 'foo.egg', 'file:///usr/']:
            self.assertRaises(AssertionError, dist_naming.split_dist, dist)

    def test_is_valid_eggname(self):
        for fn, valid in [
            ('numpy-1.3.4-7.egg', True),
            ('numpy-1.3.4n7-py2.5.egg', False),
            ('numpy-1.3.4-172.egg', True),
            ('numpy-1.3.4-py2.5-win32.egg', False),
            ]:
            self.assertEqual(dist_naming.is_valid_eggname(fn), valid)

    def test_split_eggname(self):
        for fn, nvb in [
            ('numpy-1.3.4-7.egg', ('numpy', '1.3.4', 7)),
            ('python_dateutil-0.5-12.egg', ('python_dateutil', '0.5', 12)),
            ]:
            self.assertEqual(dist_naming.split_eggname(fn), nvb)

    def test_cleanup_reponame(self):
        for repo, a in [
            ('http://www.example.com/repo', '/'),
            ('https://www.example.com/repo/', ''),
            ('file:///home/repo', '/'),
            ('file:///home/repo/', ''),
            ('file://E:\\eggs', '\\'),
            ('file://E:\\eggs\\', ''),
            ('file://C:\\Desk and Top', '\\'),
            ]:
            self.assertEqual(dist_naming.cleanup_reponame(repo), repo + a)


        sys_prefix = normpath(sys.prefix)
        self.assertEqual(dist_naming.cleanup_reponame(sys_prefix),
                         'file://' + normpath(sys_prefix) +
                         ('\\' if sys.platform == 'win32' else '/'))

    def test_comparable_spec1(self):
        cs = dist_naming.comparable_spec
        s1 = cs(dict(version='2008j', build=1))
        s2 = cs(dict(version='2008j', build=2))
        s3 = cs(dict(version='2009c', build=1))
        self.assert_(s1 < s2 < s3)

    def test_comparable_spec2(self):
        lst = []
        for v, b in [
            ('0.7.0', 1),
            ('0.8.0.dev4657', 2),
            ('0.8.0.dev5876', 1),
            ('0.8.0.dev19461', 3),
            ('0.8.0', 1),
            ]:
            lst.append(dist_naming.comparable_spec(dict(version=v, build=b)))

        for i in range(len(lst) - 1):
            self.assert_(lst[i] < lst[i + 1])


class TestReq(unittest.TestCase):

    def test_init(self):
        for req_string, name, version, build, strictness in [
            ('',          None,  None,  None, 0),
            (' \t',       None,  None,  None, 0),
            ('foo',       'foo', None,  None, 1),
            ('bar 1.9',   'bar', '1.9', None, 2),
            ('baz 1.8-2', 'baz', '1.8', 2,    3),
            ]:
            r = Req(req_string)
            self.assertEqual(r.name, name)
            self.assertEqual(r.version, version)
            self.assertEqual(r.build, build)
            self.assertEqual(r.strictness, strictness)

    def test_misc_methods(self):
        for req_string in ['', 'foo', 'bar 1.2', 'baz 2.6.7-5']:
            r = Req(req_string)
            self.assertEqual(str(r), req_string)
            self.assertEqual(r, r)
            self.assertEqual(eval(repr(r)), r)

        self.assertNotEqual(Req('foo'), Req('bar'))
        self.assertNotEqual(Req('foo 1.4'), Req('foo 1.4-5'))

    def test_matches(self):
        spec = dict(metadata_version='1.1', cname='foo_bar', version='2.4.1',
                    build=3, python=None)
        for req_string, m in [
            ('', True),
            ('foo', False),
            ('Foo-BAR', True),
            ('foo-Bar 2.4.1', True),
            ('FOO-Bar 1.8.7', False),
            ('FOO-BAR 2.4.1-3', True),
            ('FOO-Bar 2.4.1-1', False),
            ]:
            self.assertEqual(Req(req_string).matches(spec), m, req_string)

    def test_matches_py(self):
        spec = dict(metadata_version='1.1', cname='foo', version='2.4.1',
                    build=3, python=None)
        for py in ['2.4', '2.5', '2.6', '3.1']:
            requirement.PY_VER = py
            self.assertEqual(Req('foo').matches(spec), True)

        spec25 = dict(spec)
        spec25.update(dict(python='2.5'))

        spec26 = dict(spec)
        spec26.update(dict(python='2.6'))

        requirement.PY_VER = '2.5'
        self.assertEqual(Req('foo').matches(spec25), True)
        self.assertEqual(Req('foo').matches(spec26), False)

        requirement.PY_VER = '2.6'
        self.assertEqual(Req('foo').matches(spec25), False)
        self.assertEqual(Req('foo').matches(spec26), True)

    def test_dist_as_req(self):
        for req_string, s in [
            ('numpy', 1),
            ('numpy 1.3.0', 2),
            ('numpy 1.3.0-2', 3),
            ]:
            req = dist_as_req('file:///numpy-1.3.0-2.egg', s)
            self.assertEqual(req, Req(req_string))
            self.assertEqual(req.strictness, s)

    def test_add_Reqs_to_spec(self):
        spec = dict(name='dummy', packages=[])
        add_Reqs_to_spec(spec)
        self.assertEqual(spec['Reqs'], set())

        spec = dict(name='dumy', packages=['numpy 1.3.0'])
        add_Reqs_to_spec(spec)
        Reqs = spec['Reqs']
        self.assertEqual(len(Reqs), 1)
        self.assertEqual(Reqs, set([Req('numpy 1.3.0')]))



def eggs_rs(c, req_string):
    return [dist_naming.filename_dist(d)
            for d in c.install_sequence(Req(req_string))]


class TestChain0(unittest.TestCase):
    def setUp(self):
        c = Chain(verbose=0)
        for fn in ['index-add.txt', 'index-5.1.txt', 'index-5.0.txt']:
            repo = "{0}/".format(path_to_uri(HERE_DIR))
            c.add_repo(repo, fn)

        self.c = c

    @mock.patch("enstaller.indexed_repo.requirement.PY_VER", "2.5")
    def test_25(self):
        self.assertEqual(eggs_rs(self.c, 'SciPy 0.8.0.dev5698'),
                         ['freetype-2.3.7-1.egg', 'libjpeg-7.0-1.egg',
                          'numpy-1.3.0-1.egg', 'PIL-1.1.6-4.egg',
                          'scipy-0.8.0.dev5698-1.egg'])

        self.assertEqual(eggs_rs(self.c, 'SciPy'),
                         ['numpy-1.3.0-1.egg', 'scipy-0.8.0-1.egg'])

        self.assertEqual(eggs_rs(self.c, 'epdcore'),
                         ['AppInst-2.0.4-1.egg', 'numpy-1.3.0-1.egg',
                          'scipy-0.8.0-1.egg', 'EPDCore-1.2.5-1.egg'])

    @mock.patch("enstaller.indexed_repo.requirement.PY_VER", "2.6")
    def test_26(self):
        self.assertEqual(eggs_rs(self.c, 'SciPy'),
                         ['numpy-1.3.0-2.egg', 'scipy-0.8.0-2.egg'])

        self.assertEqual(eggs_rs(self.c, 'epdcore'),
                         ['numpy-1.3.0-2.egg', 'scipy-0.8.0-2.egg',
                          'EPDCore-2.0.0-1.egg'])

class TestChain1(unittest.TestCase):
    def setUp(self):
        repos = {None: None}
        c = Chain(verbose=0)
        for name in 'epd', 'gpl':
            # XXX: relying on having a '/' is horrible, but that assumption is made
            # in enough places through the code that we don't want to change it.
            repo = "{0}/".format(path_to_uri(posixpath.join(HERE_DIR, name)))
            c.add_repo(repo, 'index-7.1.txt')
            repos[name] = repo
        self.c = c
        self.repos = repos

    @mock.patch("enstaller.indexed_repo.requirement.PY_VER", "2.7")
    def test_get_repo(self):
        for req_string, repo_name in [
            ('MySQL_python', 'gpl'),
            ('bitarray', 'epd'),
            ('foobar', None),
            ]:
            self.assertEqual(self.c.get_repo(Req(req_string)),
                             self.repos[repo_name])

    @mock.patch("enstaller.indexed_repo.requirement.PY_VER", "2.7")
    def test_get_dist(self):
        for req_string, dist in [
            ('MySQL_python',  self.repos['gpl'] + 'MySQL_python-1.2.3-2.egg'),
            ('numpy',         self.repos['epd'] + 'numpy-1.6.0-3.egg'),
            ('swig',          self.repos['epd'] + 'swig-1.3.40-2.egg'),
            ('swig 1.3.36',   self.repos['epd'] + 'swig-1.3.36-3.egg'),
            ('swig 1.3.40-1', self.repos['epd'] + 'swig-1.3.40-1.egg'),
            ('swig 1.3.40-2', self.repos['epd'] + 'swig-1.3.40-2.egg'),
            ('foobar', None),
            ]:
            self.assertEqual(self.c.get_dist(Req(req_string)), dist)

    def test_reqs_dist(self):
        dist = self.repos['epd'] + 'FiPy-2.1-1.egg'
        self.assertEqual(self.c.reqs_dist(dist),
                         set([Req('distribute'),
                              Req('scipy'),
                              Req('numpy'),
                              Req('pysparse 1.2.dev203')]))

    @mock.patch("enstaller.indexed_repo.requirement.PY_VER", "2.7")
    def test_root(self):
        self.assertEqual(self.c.install_sequence(Req('numpy 1.5.1'),
                                                 mode='root'),
                         [self.repos['epd'] + 'numpy-1.5.1-2.egg'])

        self.assertEqual(self.c.install_sequence(Req('numpy 1.5.1-1'),
                                                 mode='root'),
                         [self.repos['epd'] + 'numpy-1.5.1-1.egg'])

    @mock.patch("enstaller.indexed_repo.requirement.PY_VER", "2.7")
    def test_order1(self):
        self.assertEqual(self.c.install_sequence(Req('numpy')),
                         [self.repos['epd'] + egg for egg in
                          ('MKL-10.3-1.egg', 'numpy-1.6.0-3.egg')])

    @mock.patch("enstaller.indexed_repo.requirement.PY_VER", "2.7")
    def test_order2(self):
        self.assertEqual(self.c.install_sequence(Req('scipy')),
                         [self.repos['epd'] + egg for egg in
                          ('MKL-10.3-1.egg', 'numpy-1.5.1-2.egg',
                          'scipy-0.9.0-1.egg')])


class TestChain2(unittest.TestCase):
    def setUp(self):
        repos = {}
        c = Chain(verbose=0)
        for name in 'open', 'runner', 'epd':
            repo = "{0}/".format(path_to_uri(posixpath.join(HERE_DIR, name)))
            c.add_repo(repo, 'index-7.1.txt')
            repos[name] = repo
        self.c = c
        self.repos = repos

    @mock.patch("enstaller.indexed_repo.requirement.PY_VER", "2.7")
    def test_flat_recur1(self):
        d1 = self.c.install_sequence(Req('openepd'), mode='flat')
        d2 = self.c.install_sequence(Req('openepd'), mode='recur')
        self.assertEqual(d1, d2)
        d3 = self.c.install_sequence(Req('foo'), mode='recur')
        self.assertEqual(d2[:-1], d3[:-1])

    def test_flat_recur2(self):
        for rs in 'epd 7.0', 'epd 7.0-1', 'epd 7.0-2':
            d1 = self.c.install_sequence(Req(rs), mode='flat')
            d2 = self.c.install_sequence(Req(rs), mode='recur')
            self.assertEqual(d1, d2)

    @mock.patch("enstaller.indexed_repo.requirement.PY_VER", "2.7")
    def test_multiple_reqs(self):
        lst = self.c.install_sequence(Req('ets'))
        self.assert_(self.repos['epd'] + 'numpy-1.5.1-2.egg' in lst)


if __name__ == '__main__':
    unittest.main()



import io

from enstaller.vendor.requests.structures import CaseInsensitiveDict

from .compat import HTTPResponse, pickle


class Serializer(object):

    def dumps(self, request, response, body=None):
        response_headers = CaseInsensitiveDict(response.headers)

        if body is None:
            body = response.read(decode_content=False)
            response._fp = io.BytesIO(body)

        data = {
            "response": {
                "body": body,
                "headers": response.headers,
                "status": response.status,
                "version": response.version,
                "reason": response.reason,
                "strict": response.strict,
                "decode_content": response.decode_content,
            },
        }

        # Construct our vary headers
        data["vary"] = {}
        if "vary" in response_headers:
            varied_headers = response_headers['vary'].split(',')
            for header in varied_headers:
                header = header.strip()
                data["vary"][header] = request.headers.get(header, None)

        return b"cc=1," + pickle.dumps(data, pickle.HIGHEST_PROTOCOL)

    def loads(self, request, data):
        # Short circuit if we've been given an empty set of data
        if not data:
            return

        # Determine what version of the serializer the data was serialized
        # with
        try:
            ver, data = data.split(b",", 1)
        except ValueError:
            ver = b"cc=0"

        # Make sure that our "ver" is actually a version and isn't a false
        # positive from a , being in the data stream.
        if ver[:3] != b"cc=":
            data = ver + data
            ver = b"cc=0"

        # Get the version number out of the cc=N
        ver = ver.split(b"=", 1)[-1].decode("ascii")

        # Dispatch to the actual load method for the given version
        try:
            return getattr(self, "_loads_v{0}".format(ver))(request, data)
        except AttributeError:
            # This is a version we don't have a loads function for, so we'll
            # just treat it as a miss and return None
            return

    def _loads_v0(self, request, data):
        # The original legacy cache data. This doesn't contain enough
        # information to construct everything we need, so we'll treat this as
        # a miss.
        return

    def _loads_v1(self, request, data):
        try:
            cached = pickle.loads(data)
        except ValueError:
            return

        # Special case the '*' Vary value as it means we cannot actually
        # determine if the cached response is suitable for this request.
        if "*" in cached.get("vary", {}):
            return

        # Ensure that the Vary headers for the cached response match our
        # request
        for header, value in cached.get("vary", {}).items():
            if request.headers.get(header, None) != value:
                return

        body = io.BytesIO(cached["response"].pop("body"))
        return HTTPResponse(
            body=body,
            preload_content=False,
            **cached["response"]
        )



import json

from egginst.vendor.six.moves import unittest

from enstaller.config import Configuration
from enstaller.session import Session
from enstaller.tests.common import R_JSON_AUTH_RESP
from enstaller.auth.user_info import UserInfo
from enstaller.vendor import responses


FAKE_AUTH = ("nono", "le petit robot")


class TestUserInfo(unittest.TestCase):
    @responses.activate
    def test_simple_legacy_canopy(self):
        # Given
        url = "https://api.enthought.com"

        config = Configuration()
        config.update(store_url=url, auth=FAKE_AUTH)

        responses.add(responses.GET, url + "/accounts/user/info/", status=200,
                      body=json.dumps(R_JSON_AUTH_RESP))

        session = Session.from_configuration(config)
        session.authenticate(config.auth)

        # When
        user_info = UserInfo.from_session(session)

        # Then
        self.assertEqual(user_info.first_name, R_JSON_AUTH_RESP["first_name"])

    @responses.activate
    def test_simple_old_legacy(self):
        # Given
        url = "https://acme.com"

        config = Configuration()
        config.update(use_webservice=False, auth=FAKE_AUTH)
        config.update(indexed_repositories=[url])

        responses.add(responses.HEAD, config.indices[0][0], status=200)

        session = Session.from_configuration(config)
        session.authenticate(config.auth)

        # When
        user_info = UserInfo.from_session(session)

        # Then
        self.assertTrue(user_info.is_authenticated)

    @responses.activate
    def test_simple_brood_auth(self):
        # Given
        url = "https://acme.com"
        token_url = url + "/api/v0/json/auth/tokens/auth"

        config = Configuration()
        config.update(store_url="brood+" + url, auth=FAKE_AUTH)

        responses.add(responses.POST, token_url, status=200,
                      body=json.dumps({"token": "dummy token"}))

        session = Session.from_configuration(config)
        session.authenticate(config.auth)

        # When
        user_info = UserInfo.from_session(session)

        # Then
        self.assertTrue(user_info.is_authenticated)


class TestSubscriptionLevel(unittest.TestCase):
    def test_unsubscribed_user(self):
        user_info = UserInfo(True)
        self.assertEqual(user_info.subscription_level, "Canopy / EPD Free")

        user_info = UserInfo(False)
        self.assertIsNone(user_info.subscription_level)

    def test_subscribed_user(self):
        user_info = UserInfo(True, has_subscription=True)
        self.assertEqual(user_info.subscription_level, "Canopy / EPD Basic or above")

        user_info = UserInfo(True, has_subscription=False)
        self.assertEqual(user_info.subscription_level, "Canopy / EPD Free")

        user_info = UserInfo(False, has_subscription=False)
        self.assertIsNone(user_info.subscription_level)



"""
escape/unescape routines available for backends which need
alphanumeric usernames, services, or other values
"""

import re
import string
import sys

# True if we are running on Python 3.
# taken from six.py
PY3 = sys.version_info[0] == 3

# allow use of unicode literals
if PY3:
    def u(s):
        return s
    def _unichr(c):
        return chr(c)
else:
    def u(s):
        return unicode(s, "unicode_escape")
    def _unichr(c):
        return unichr(c)

LEGAL_CHARS = (
    getattr(string, 'letters', None)  # Python 2
    or getattr(string, 'ascii_letters')  # Python 3
) + string.digits

ESCAPE_FMT = "_%02X"

def _escape_char(c):
    "Single char escape. Return the char, escaped if not already legal"
    if isinstance(c, int):
        c = _unichr(c)
    return c if c in LEGAL_CHARS else ESCAPE_FMT % ord(c)

def escape(value):
    """
    Escapes given string so the result consists of alphanumeric chars and
    underscore only.
    """
    return "".join(_escape_char(c) for c in value.encode('utf-8'))

def _unescape_code(regex_match):
    ordinal = int(regex_match.group('code'), 16)
    if sys.version_info >= (3,):
        return bytes([ordinal])
    return chr(ordinal)

def unescape(value):
    """
    Inverse of escape.
    """
    re_esc = re.compile(
        # the pattern must be bytes to operate on bytes
        ESCAPE_FMT.replace('%02X', '(?P<code>[0-9A-F]{2})').encode('ascii')
    )
    return re_esc.sub(_unescape_code, value.encode('ascii')).decode('utf-8')



import os
import tempfile

from egginst.vendor.six.moves import unittest

from egginst._post_install_lib import safe_write, update_pkg_config_prefix


class TestSafeWrite(unittest.TestCase):
    def setUp(self):
        f = tempfile.NamedTemporaryFile(delete=False)
        self.filename = f.name
        f.close()

    def tearDown(self):
        os.remove(self.filename)

    def test_simple(self):
        """Test whether the file content is correctly written when using string."""
        name = self.filename
        safe_write(name, "foo")

        with open(name) as fp:
            self.assertEqual(fp.read(), "foo")

    def test_already_exists(self):
        """Test whether safe_write works when the target already exists."""
        safe_write(self.filename, "foo")
        safe_write(self.filename, "foo")

        with open(self.filename) as fp:
            self.assertEqual(fp.read(), "foo")

    def test_simple_callable(self):
        """Test whether the file content is correctly written when using callable."""
        name = self.filename
        safe_write(name, lambda fp: fp.write("foo"))

        with open(name) as fp:
            self.assertEqual(fp.read(), "foo")

    def test_simple_error(self):
        """Test whether the file content is not half written if error happens."""
        name = self.filename
        safe_write(name, "foo")

        def simulate_interrupted(fp):
            fp.write("bar")
            raise KeyboardInterrupt()
            fp.write("foo")

        self.assertRaises(KeyboardInterrupt, safe_write, name, simulate_interrupted)

        with open(name) as fp:
            self.assertEqual(fp.read(), "foo")


class TestUpdatePkgConfig(unittest.TestCase):
    def setUp(self):
        f = tempfile.NamedTemporaryFile(delete=False)
        self.filename = f.name
        f.close()

    def tearDown(self):
        os.remove(self.filename)

    def test_simple(self):
        """Test whether the file content is correctly written when using string."""
        data = """\
prefix=/home/vagrant/pisi/tmp/Qt-4.8.2-2/usr
exec_prefix=${prefix}
libdir=${prefix}/lib
"""

        r_data = """\
prefix=/foo/bar
exec_prefix=${prefix}
libdir=${prefix}/lib
"""
        with tempfile.NamedTemporaryFile(suffix=".pc",
                                         delete=False, mode="wt") as fp:
            pc_file = fp.name
            fp.write(data)
            fp.close()

            update_pkg_config_prefix(pc_file, "/foo/bar")

            with open(pc_file) as r_fp:
                self.assertEqual(r_fp.read(), r_data)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function, absolute_import 

import os
import sys
import hashlib
import zipfile
from collections import defaultdict
from os.path import basename, isfile, isdir, join

from egginst.utils import human_bytes

from enstaller.utils import comparable_version, compute_md5, uri_to_path
from . import metadata
from . import dist_naming
from .requirement import Req, add_Reqs_to_spec


def stream_to_file(fi, path, info={}):
    """
    Read data from the filehandle and write a the file.
    Optionally check the MD5.
    """
    size = info['size']
    md5 = info.get('md5')

    print("Fetching: %s (%s)" % (basename(path), human_bytes(size)))

    n = 0
    h = hashlib.new('md5')
    with open(path + '.part', 'wb') as fo:
        while True:
            chunk = fi.read(16384)
            if not chunk:
                break
            fo.write(chunk)
            if md5:
                h.update(chunk)
            n += len(chunk)
    fi.close()

    if md5 and h.hexdigest() != md5:
        raise Exception("Error: received data MD5 sums mismatch")
    os.rename(path + '.part', path)


class Chain(object):

    def __init__(self, repos=[], verbose=False):
        self.verbose = verbose

        # maps distributions to specs
        self.index = {}

        # maps cnames to the list of distributions (in repository order)
        self.groups = defaultdict(list)

        # maps the url of a repo to repository objects
        self.repo_objs = {}

        # Chain of repositories, either local or remote
        self.repos = []
        for repo in repos:
            # These are file:// (optionally indexed) or http:// (indexed)
            self.add_repo(repo)

        if self.verbose:
            self.print_repos()


    def print_repos(self):
        print('Repositories:')
        for r in self.repos:
            print('  %r' % r)
        print()


    def connect(self, repo):
        msg = "If you need this method, contact the enstaller maintainers."
        raise NotImplementedError(msg)

    def add_repo(self, repo, index_fn=None):
        """
        Add a repo to the chain, i.e. read the index file of the url,
        parse it and update the index.
        """
        if self.verbose:
            print("Adding repository:")
            print("   URL:", repo)

        repo = dist_naming.cleanup_reponame(repo)
        self.repos.append(repo)

        if index_fn: # for running the tests locally
            repo_path = uri_to_path(repo)
            with open(join(repo_path, index_fn), "rt") as fp:
                index_data = fp.read()
            new_index = metadata.parse_depend_index(index_data)

        else:
            new_index = dict(self.connect(repo).query(type='egg'))

        for spec in new_index.values():
            add_Reqs_to_spec(spec)

        for distname, spec in new_index.items():
            dist = repo + distname
            self.index[dist] = spec
            self.groups[spec['cname']].append(dist)


    def get_version_build(self, dist):
        """
        Returns a tuple(version, build) for a distribution, version is a
        RationalVersion object (see verlib).  This method is used below
        for determining the distribution with the largest version and build
        number.
        """
        return dist_naming.comparable_spec(self.index[dist])


    def iter_dists(self, req):
        """
        iterate over all distributions matching the requirement (in
        repository order)
        """
        assert req.strictness >= 1
        for dist in self.groups[req.name]:
            if req.matches(self.index[dist]):
                yield dist


    def get_repo(self, req):
        """
        return the first repository in which the requirement matches at least
        one distribution
        """
        for dist in self.iter_dists(req):
            return dist_naming.repo_dist(dist)
        return None


    def get_dist(self, req):
        """
        return the distributions with the largest version and build number
        from the first repository which contains any matches
        """
        repo = self.get_repo(req)
        if repo is None:
            return None

        matches = []
        for dist in self.iter_dists(req):
            if dist_naming.repo_dist(dist) == repo:
                matches.append(dist)
        return max(matches, key=self.get_version_build)


    def reqs_dist(self, dist):
        """
        return the set of requirement objects listed by the given
        distribution
        """
        return self.index[dist]['Reqs']


    def cname_dist(self, dist):
        """
        return the canonical project name for a given distribution
        """
        return self.index[dist]['cname']


    def are_complete(self, dists):
        """
        return True if the distributions 'dists' are complete, i.e. the for
        each distribution all dependencies (by name only) are also included
        in the 'dists'
        """
        cnames = set(self.cname_dist(d) for d in dists)
        for dist in dists:
            for r in self.reqs_dist(dist):
                if r.name not in cnames:
                    return False
        return True


    def determine_install_order(self, dists):
        """
        given the distributions 'dists' (which are already complete, i.e.
        the for each distribution all dependencies are also included in
        the 'dists'), return a list of the same distribution in the correct
        install order
        """
        dists = list(dists)
        assert self.are_complete(dists)

        # make sure each project name is listed only once
        assert len(dists) == len(set(self.cname_dist(d) for d in dists))

        # the distributions corresponding to the requirements must be sorted
        # because the output of this function is otherwise not deterministic
        dists.sort(key=self.cname_dist)

        # maps dist -> set of required (project) names
        rns = {}
        for dist in dists:
            rns[dist] = set(r.name for r in self.reqs_dist(dist))

        # as long as we have things missing, simply look for things which
        # can be added, i.e. all the requirements have been added already
        result = []
        names_inst = set()
        while len(result) < len(dists):
            n = len(result)
            for dist in dists:
                if dist in result:
                    continue
                # see if all required packages were added already
                if all(bool(name in names_inst) for name in rns[dist]):
                    result.append(dist)
                    names_inst.add(self.index[dist]['cname'])
                    assert len(names_inst) == len(result)

            if len(result) == n:
                # nothing was added
                raise Exception("Loop in dependency graph\n%r" %
                                [dist_naming.filename_dist(d) for d in dists])
        return result


    def _sequence_flat(self, root):
        dists = [root]
        for r in self.reqs_dist(root):
            d = self.get_dist(r)
            if d is None:
                sys.exit('Error: could not resolve %r' % r)
            dists.append(d)

        can_order = self.are_complete(dists)
        if self.verbose:
            print("Can determine install order:", can_order)
        if can_order:
            dists = self.determine_install_order(dists)
        return dists


    def _sequence_recur(self, root):
        reqs_shallow = {}
        for r in self.reqs_dist(root):
            reqs_shallow[r.name] = r
        reqs_deep = defaultdict(set)

        def add_dependents(dist):
            for r in self.reqs_dist(dist):
                reqs_deep[r.name].add(r)
                if (r.name in reqs_shallow  and
                        r.strictness < reqs_shallow[r.name].strictness):
                    continue
                d = self.get_dist(r)
                if d is None:
                    sys.exit('Error: could not resolve %r required by %r' %
                             (r, dist))
                dists.add(d)
                add_dependents(d)

        dists = set([root])
        add_dependents(root)

        cnames = set(self.cname_dist(d) for d in dists)
        if len(dists) != len(cnames):
            for cname in cnames:
                ds = [d for d in dists if self.cname_dist(d) == cname]
                assert len(ds) != 0
                if len(ds) == 1:
                    continue
                if self.verbose:
                    print('multiple: %s' % cname)
                    for d in ds:
                        print('    %s' % d)
                r = max(reqs_deep[cname], key=lambda r: r.strictness)
                assert r.name == cname
                # remove the dists with name 'cname'
                dists = [d for d in dists if self.cname_dist(d) != cname]
                # add the one
                dists.append(self.get_dist(r))

        return self.determine_install_order(dists)


    def install_sequence(self, req, mode='recur'):
        """
        Return the list of distributions which need to be installed.
        The returned list is given in dependency order.
        The 'mode' may be:

        'root':  only the distribution for the requirement itself is
                 contained in the result (but not any dependencies)

        'flat':  dependencies are handled only one level deep

        'recur': dependencies are handled recursively (default)
        """
        if self.verbose:
            print("Determining install sequence for %r" % req)
        root = self.get_dist(req)
        if root is None:
            return None

        if mode == 'root':
            return [root]

        if mode == 'flat':
            return self._sequence_flat(root)

        if mode == 'recur':
            return self._sequence_recur(root)

        raise Exception('did not expect: mode = %r' % mode)


    def list_versions(self, name):
        """
        given the name of a package, retruns a sorted list of versions for
        package `name` found in any repo.
        """
        versions = set()

        req = Req(name)
        for dist in self.groups[req.name]:
            spec = self.index[dist]
            if req.matches(spec):
                versions.add(spec['version'])

        try:
            return sorted(versions, key=comparable_version)
        except TypeError:
            return list(versions)


    def fetch_dist(self, dist, fetch_dir, force=False, dry_run=False):
        """
        Get a distribution, i.e. copy or download the distribution into
        fetch_dir.

        force:
            force download or copy if MD5 mismatches
        """
        info = self.index[dist]
        repo, fn = dist_naming.split_dist(dist)
        path = join(fetch_dir, fn)
        # if force is used, make sure the md5 is the expected, otherwise
        # only see if the file exists
        if isfile(path) and (not force or compute_md5(path) == info.get('md5')):
            if self.verbose:
                print("Not forcing refetch, %r already matches MD5" % path)
            return

        if dry_run:
            return

        if self.verbose:
            print("Fetching: %r" % dist)
            print("      to: %r" % path)

        stream_to_file(self.repo_objs[repo].get_data(fn), path, info)


    def index_file(self, filename, repo):
        """
        Add an unindexed distribution, which must already exist in a local
        repository to the index (in memory).  Note that the index file on
        disk remains unchanged.
        """
        assert filename == basename(filename), filename
        dist = repo + filename
        if self.verbose:
            print("Adding %r to index" % dist)

        arcname = 'EGG-INFO/spec/depend'
        z = zipfile.ZipFile(join(dist_naming.dirname_repo(repo), filename))
        if arcname not in z.namelist():
            z.close()
            raise Exception("zipfile %r has no arcname=%r" %
                            (filename, arcname))

        spec = metadata.parse_data(z.read(arcname))
        z.close()
        add_Reqs_to_spec(spec)
        self.index[dist] = spec
        self.groups[spec['cname']].append(dist)


    def index_all_files(self, repo):
        """
        Add all distributions to the index, see index_file() above.
        Note that no index file is written to disk.
        """
        dir_path = dist_naming.dirname_repo(repo)
        assert isdir(dir_path), dir_path
        for fn in os.listdir(dir_path):
            if not fn.endswith('.egg'):
                continue
            if not dist_naming.is_valid_eggname(fn):
                print("WARNING: ignoring invalid egg name:", join(dir_path, fn))
                continue
            self.index_file(fn, repo)


def main():
    from optparse import OptionParser

    p = OptionParser(usage="usage: %prog [options] REPO [EGG ...]",
                     description="simple interface to fetch eggs")
    p.add_option("--dst",
                 action="store",
                 help="destination directory",
                 default=os.getcwd(),
                 metavar='PATH')
    p.add_option('-v', "--verbose", action="store_true")

    opts, args = p.parse_args()

    if len(args) < 1:
        p.error('at least one argument expected, try -h')

    repo = args[0]
    c = Chain([repo], opts.verbose)
    for fn in args[1:]:
        if not dist_naming.is_valid_eggname(fn):
            sys.exit('Error: invalid egg name: %r' % fn)
        c.fetch_dist(repo + fn, opts.dst)


if __name__ == '__main__':
    sys.exit(main())



import re
import os
import json
import zipfile
from os.path import getmtime, isfile, join

from egginst.eggmeta import info_from_z

from enstaller.utils import info_file


egg_fmt = '%(name)s-%(version)s-%(build)d.egg'

egg_pat = re.compile(r'([\w.]+)-([\w.]+)-(\d+)\.egg$')


def is_valid_eggname(eggname):
    return bool(egg_pat.match(eggname))


def split_eggname(eggname):
    m = egg_pat.match(eggname)
    assert m, eggname
    return m.group(1), m.group(2), int(m.group(3))


def info_from_egg(path):
    z = zipfile.ZipFile(path)
    res = info_from_z(z)
    z.close()
    return res


def update_index(dir_path, force=False, verbose=False):
    index_path = join(dir_path, 'index.json')
    if force or not isfile(index_path):
        index = {}
    else:
        index = json.load(open(index_path, 'r'))

    new_index = {}
    for fn in os.listdir(dir_path):
        if not is_valid_eggname(fn):
            continue
        path = join(dir_path, fn)
        info = index.get(fn)
        if info and getmtime(path) == info['mtime']:
            new_index[fn] = info
            continue
        info = info_file(path)
        info.update(info_from_egg(path))
        new_index[fn] = info

    patches_index_path = join(dir_path, 'patches', 'index.json')
    if isfile(patches_index_path):
        patch_index = json.load(open(patches_index_path))
        for info in patch_index.itervalues():
            info['type'] = 'patch'
        new_index.update(patch_index)

    with open(index_path, 'w') as f:
        json.dump(new_index, f, indent=2, sort_keys=True)



import errno
import json
import logging
import ntpath
import os.path
import posixpath
import re
import shutil
import site
import sys
import tempfile
import textwrap

import mock

from egginst.tests.common import mkdtemp, DUMMY_EGG
from egginst.utils import ensure_dir
from egginst.vendor.six import StringIO
from egginst.vendor.six.moves import unittest


from enstaller.auth import UserInfo
from enstaller.config import Configuration
from enstaller.errors import InvalidPythonPathConfiguration
from enstaller.main import (check_prefixes, ensure_authenticated_config,
                            epd_install_confirm, env_option,
                            get_package_path, imports_option, install_req,
                            main, needs_to_downgrade_enstaller,
                            repository_factory, search, setup_proxy_or_die,
                            update_enstaller, _ensure_config_path,
                            _get_enstaller_comparable_version)
from enstaller.main import HOME_ENSTALLER4RC
from enstaller.eggcollect import meta_info_from_prefix
from enstaller.plat import custom_plat
from enstaller.repository import Repository, InstalledPackageMetadata
from enstaller.session import Session
from enstaller.solver import Requirement
from enstaller.utils import PY_VER
from enstaller.vendor import responses
from enstaller.versions.enpkg import EnpkgVersion

import enstaller.tests.common
from .common import (authenticated_config, create_prefix_with_eggs,
                     dummy_installed_package_factory,
                     dummy_repository_package_factory, exception_code,
                     mock_print, mock_index, mock_raw_input, fake_keyring,
                     mocked_session_factory, FakeOptions,
                     R_JSON_AUTH_FREE_RESP, R_JSON_NOAUTH_RESP,
                     DummyAuthenticator)


class TestEnstallerUpdate(unittest.TestCase):
    @mock.patch("enstaller.__is_released__", True)
    def test_no_update_enstaller(self):
        config = Configuration()
        session = mocked_session_factory(config.repository_cache)

        opts = mock.Mock()
        opts.yes = False

        self.assertFalse(update_enstaller(session, Repository(), opts))

    def _test_update_enstaller(self, low_version, high_version):
        config = Configuration()
        session = mocked_session_factory(config.repository_cache)

        enstaller_eggs = [
            dummy_repository_package_factory("enstaller", low_version, 1),
            dummy_repository_package_factory("enstaller", high_version, 1),
        ]
        repository = enstaller.tests.common.repository_factory(enstaller_eggs)

        with mock_raw_input("yes"):
            with mock.patch("enstaller.main.inplace_update", lambda *args: None):
                opts = mock.Mock()
                opts.yes = False
                return update_enstaller(session, repository, opts)

    @mock.patch("enstaller.__version__", "4.6.3")
    @mock.patch("enstaller.__is_released__", True)
    def test_update_enstaller_higher_available(self):
        # low/high versions are below/above any realistic enstaller version
        low_version, high_version = "1.0.0", "666.0.0"
        self.assertTrue(self._test_update_enstaller(low_version, high_version))

    @mock.patch("enstaller.__version__", "4.6.3")
    @mock.patch("enstaller.__is_released__", True)
    def test_update_enstaller_higher_unavailable(self):
        # both low/high versions are below current enstaller version
        low_version, high_version = "1.0.0", "2.0.0"
        self.assertFalse(self._test_update_enstaller(low_version, high_version))


class TestMisc(unittest.TestCase):
    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_epd_install_confirm(self):
        for allowed_yes in ("y", "Y", "yes", "YES", "YeS"):
            with mock_raw_input(allowed_yes):
                self.assertTrue(epd_install_confirm())

        for non_yes in ("n", "N", "no", "NO", "dummy"):
            with mock_raw_input(non_yes):
                self.assertFalse(epd_install_confirm())

    @mock.patch("sys.platform", "linux2")
    def test_get_package_path_unix(self):
        prefix = "/foo"
        r_site_packages = posixpath.join(prefix, "lib", "python" + PY_VER, "site-packages")

        self.assertEqual(get_package_path(prefix), r_site_packages)

    @mock.patch("sys.platform", "win32")
    def test_get_package_path_windows(self):
        prefix = "c:\\foo"
        r_site_packages = ntpath.join(prefix, "lib", "site-packages")

        self.assertEqual(get_package_path(prefix), r_site_packages)

    @mock.patch("sys.platform", "linux2")
    def test_check_prefixes_unix(self):
        prefixes = ["/foo", "/bar"]
        site_packages = [posixpath.join(prefix,
                                        "lib/python{0}/site-packages".
                                        format(PY_VER))
                         for prefix in prefixes]

        with mock.patch("sys.path", site_packages):
            check_prefixes(prefixes)

        with mock.patch("sys.path", site_packages[::-1]):
            with self.assertRaises(InvalidPythonPathConfiguration) as e:
                check_prefixes(prefixes)
            message = str(e.exception)
            self.assertEqual(message,
                             "Order of path prefixes doesn't match PYTHONPATH")

        with mock.patch("sys.path", []):
            with self.assertRaises(InvalidPythonPathConfiguration) as e:
                check_prefixes(prefixes)
            message = str(e.exception)
            self.assertEqual(message,
                             "Expected to find {0} in PYTHONPATH".
                             format(site_packages[0]))

    @mock.patch("sys.platform", "win32")
    def test_check_prefixes_win32(self):
        prefixes = ["c:\\foo", "c:\\bar"]
        site_packages = [ntpath.join(prefix, "lib", "site-packages")
                         for prefix in prefixes]

        with mock.patch("sys.path", site_packages):
            check_prefixes(prefixes)

        with mock.patch("sys.path", site_packages[::-1]):
            with self.assertRaises(InvalidPythonPathConfiguration) as e:
                check_prefixes(prefixes)
            message = str(e.exception)
            self.assertEqual(message, "Order of path prefixes doesn't match PYTHONPATH")

        with mock.patch("sys.path", []):
            with self.assertRaises(InvalidPythonPathConfiguration) as e:
                check_prefixes(prefixes)
            message = str(e.exception)
            self.assertEqual(message,
                             "Expected to find {0} in PYTHONPATH".
                             format(site_packages[0]))

    def test_imports_option_empty(self):
        # Given
        r_output = textwrap.dedent("""\
            Name                 Version              Location
            ============================================================
            """)
        repository = Repository()

        # When
        with mock_print() as m:
            imports_option(repository)

        # Then
        self.assertMultiLineEqual(m.value, r_output)

    def test_imports_option_sys_only(self):
        # Given
        r_output = textwrap.dedent("""\
            Name                 Version              Location
            ============================================================
            dummy                1.0.1-1              sys
            """)

        repository = Repository()
        metadata = InstalledPackageMetadata.from_egg(DUMMY_EGG,
                                                     "random string",
                                                     sys.prefix)
        repository.add_package(metadata)

        # When
        with mock_print() as m:
            imports_option(repository)

        # Then
        self.assertMultiLineEqual(m.value, r_output)

    def test_env_options(self):
        # Given
        prefix = sys.prefix
        r_output = textwrap.dedent("""\
            Prefixes:
                {0} (sys)
        """.format(prefix))

        # When
        with mock_print() as m:
            env_option([sys.prefix])

        # Then
        self.assertMultiLineEqual(m.value, r_output)

    def test_env_options_multiple_prefixes(self):
        # Given
        if sys.platform == "win32":
            prefixes = ["C:/opt", sys.prefix]
        else:
            prefixes = ["/opt", sys.prefix]
        r_output = textwrap.dedent("""\
            Prefixes:
                {0}
                {1} (sys)
        """.format(prefixes[0], prefixes[1]))

        # When
        with mock_print() as m:
            env_option(prefixes)

        # Then
        self.assertMultiLineEqual(m.value, r_output)

    def test_needs_to_downgrade(self):
        # Given
        reqs = []

        # When/Then
        self.assertFalse(needs_to_downgrade_enstaller(reqs))

        # Given
        reqs = [Requirement.from_anything("numpy"),
                Requirement.from_anything("scipy")]

        # When/Then
        self.assertFalse(needs_to_downgrade_enstaller(reqs))

        # Given
        reqs = [Requirement.from_anything("enstaller"),
                Requirement.from_anything("scipy")]

        # When/Then
        self.assertFalse(needs_to_downgrade_enstaller(reqs))

        # Given
        reqs = [Requirement.from_anything("enstaller 4.5.1")]

        # When/Then
        self.assertTrue(needs_to_downgrade_enstaller(reqs))

    @mock.patch("enstaller.main.write_default_config")
    def test_get_config_filename_nothing_anywhere(self, write_default_config):
        # When
        with mock.patch("enstaller.main._get_config_candidate", return_value=None):
            config_path = _ensure_config_path()

        # Then
        self.assertEqual(config_path, HOME_ENSTALLER4RC)
        self.assertTrue(write_default_config.called_with(config_path,
                                                         HOME_ENSTALLER4RC))

    @mock.patch("enstaller.main.write_default_config")
    def test_get_config_filename_nothing_in_sys_prefix(self, write_default_config):
        # Given
        prefix = self.tempdir

        # When
        with mock.patch("enstaller.config.sys.prefix", prefix):
            config_path = _ensure_config_path()

        # Then
        self.assertEqual(config_path, HOME_ENSTALLER4RC)

    @mock.patch("enstaller.main.write_default_config")
    def test_get_config_filename_file_in_sys_prefix(self, write_default_config):
        # Given
        prefix = self.tempdir
        path = os.path.join(prefix, ".enstaller4rc")

        with open(path, "w") as fp:
            fp.write("")

        # When
        with mock.patch("enstaller.config.sys.prefix", prefix):
            config_path = _ensure_config_path()

        # Then
        self.assertEqual(config_path, path)

    def _mock_index(self, entries):
        index = dict((entry.key, entry.s3index_data) for entry in entries)

        responses.add(responses.GET,
                      "https://api.enthought.com/eggs/{0}/index.json".format(custom_plat),
                      body=json.dumps(index), status=200,
                      content_type='application/json')

    @responses.activate
    def test_repository_factory(self):
        # Given
        config = Configuration()
        entries = [
            dummy_repository_package_factory("numpy", "1.8.0", 1),
            dummy_repository_package_factory("scipy", "0.13.3", 1),
        ]
        self._mock_index(entries)

        # When
        with Session(DummyAuthenticator(), self.tempdir) as session:
            repository = repository_factory(session, config.indices)

        # Then
        repository.find_package("numpy", "1.8.0-1")
        repository.find_package("scipy", "0.13.3-1")

        self.assertEqual(repository.find_packages("nose"), [])

    def test_setup_proxy_or_die(self):
        # Given
        proxy_string = "http://acme.com:3128"
        config = Configuration()

        # When
        setup_proxy_or_die(config, proxy_string)

        # Then
        self.assertEqual(config.proxy_dict, {"http": "http://acme.com:3128"})

    @unittest.skipIf(sys.version_info < (2, 7),
                     "Bug in 2.6 stdlib for parsing url without scheme")
    def test_setup_proxy_or_die_without_scheme(self):
        # Given
        proxy_string = "acme.com:3128"
        config = Configuration()

        # When
        setup_proxy_or_die(config, proxy_string)

        # Then
        self.assertEqual(config.proxy_dict, {"http": "http://acme.com:3128"})

        # Given
        proxy_string = ":3128"
        config = Configuration()

        # When/Then
        with self.assertRaises(SystemExit) as exc:
            setup_proxy_or_die(config, proxy_string)
        self.assertEqual(exc.exception.code, 1)


class TestSearch(unittest.TestCase):
    def test_no_installed(self):
        config = Configuration()
        config.update(use_webservice=False)

        with mkdtemp() as d:
            # XXX: isn't there a better way to ensure ws at the end of a line
            # are not eaten away ?
            r_output = textwrap.dedent("""\
                Name                   Versions           Product              Note
                ================================================================================
                another_dummy          2.0.0-1            commercial           {0}
                dummy                  0.9.8-1            commercial           {0}
                                       1.0.0-1            commercial           {0}
                """.format(""))
            entries = [dummy_repository_package_factory("dummy", "1.0.0", 1),
                       dummy_repository_package_factory("dummy", "0.9.8", 1),
                       dummy_repository_package_factory("another_dummy", "2.0.0", 1)]
            enpkg = create_prefix_with_eggs(config, d, remote_entries=entries)

            with mock_print() as m:
                search(enpkg._remote_repository,
                       enpkg._top_installed_repository, config, UserInfo(True))
                self.assertMultiLineEqual(m.value, r_output)

    def test_installed(self):
        config = Configuration()
        config.update(use_webservice=False)

        with mkdtemp() as d:
            r_output = textwrap.dedent("""\
                Name                   Versions           Product              Note
                ================================================================================
                dummy                  0.9.8-1            commercial           {0}
                                     * 1.0.1-1            commercial           {0}
                """.format(""))
            entries = [dummy_repository_package_factory("dummy", "1.0.1", 1),
                       dummy_repository_package_factory("dummy", "0.9.8", 1)]
            installed_entries = [dummy_installed_package_factory("dummy", "1.0.1", 1)]
            enpkg = create_prefix_with_eggs(config, d, installed_entries, entries)

            with mock_print() as m:
                search(enpkg._remote_repository,
                       enpkg._installed_repository, config, UserInfo(True))
                self.assertMultiLineEqual(m.value, r_output)

    def test_pattern(self):
        config = Configuration()
        config.update(use_webservice=False)
        with mkdtemp() as d:
            r_output = textwrap.dedent("""\
                Name                   Versions           Product              Note
                ================================================================================
                dummy                  0.9.8-1            commercial           {0}
                                     * 1.0.1-1            commercial           {0}
                """.format(""))
            entries = [dummy_repository_package_factory("dummy", "1.0.1", 1),
                       dummy_repository_package_factory("dummy", "0.9.8", 1),
                       dummy_repository_package_factory("another_package", "2.0.0", 1)]
            installed_entries = [dummy_installed_package_factory("dummy", "1.0.1", 1)]
            enpkg = create_prefix_with_eggs(config, d, installed_entries, entries)

            with mock_print() as m:
                search(enpkg._remote_repository,
                       enpkg._top_installed_repository,
                       config, UserInfo(True),
                       pat=re.compile("dummy"))
                self.assertMultiLineEqual(m.value, r_output)

            r_output = textwrap.dedent("""\
                Name                   Versions           Product              Note
                ================================================================================
                another_package        2.0.0-1            commercial           {0}
                dummy                  0.9.8-1            commercial           {0}
                                     * 1.0.1-1            commercial           {0}
                """.format(""))
            with mock_print() as m:
                search(enpkg._remote_repository,
                       enpkg._top_installed_repository, config,
                       UserInfo(True), pat=re.compile(".*"))
                self.assertMultiLineEqual(m.value, r_output)

    @responses.activate
    def test_not_available(self):
        responses.add(responses.GET,
                      "https://acme.com/accounts/user/info/",
                      body=json.dumps(R_JSON_AUTH_FREE_RESP))
        config = Configuration()
        config.update(store_url="https://acme.com")

        r_output = textwrap.dedent("""\
            Name                   Versions           Product              Note
            ================================================================================
            another_package        2.0.0-1            commercial           not subscribed to
            dummy                  0.9.8-1            commercial           {0}
                                   1.0.1-1            commercial           {0}
            Note: some of those packages are not available at your current
            subscription level ('Canopy / EPD Free').
            """.format(""))
        another_entry = dummy_repository_package_factory("another_package", "2.0.0", 1)
        another_entry.available = False

        entries = [dummy_repository_package_factory("dummy", "1.0.1", 1),
                   dummy_repository_package_factory("dummy", "0.9.8", 1),
                   another_entry]

        with Session.from_configuration(config) as session:
            with mkdtemp() as d:
                with mock_print() as m:
                    enpkg = create_prefix_with_eggs(config, d, remote_entries=entries)
                    search(enpkg._remote_repository,
                           enpkg._installed_repository, config, session)

                self.assertMultiLineEqual(m.value, r_output)


@fake_keyring
class TestInstallRequirement(unittest.TestCase):
    def setUp(self):
        self.prefix = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.prefix)

    @mock.patch("sys.platform", "darwin")
    def test_os_error_darwin(self):
        config = Configuration()

        remote_entries = [
            dummy_repository_package_factory("nose", "1.3.0", 1)
        ]

        with mock.patch("enstaller.main.Enpkg.execute") as m:
            error = OSError()
            error.errno = errno.EACCES
            m.side_effect = error
            enpkg = create_prefix_with_eggs(config, self.prefix, [], remote_entries)
            with self.assertRaises(SystemExit):
                install_req(enpkg, config, "nose", FakeOptions())

    @mock.patch("sys.platform", "linux2")
    def test_os_error(self):
        config = Configuration()

        remote_entries = [
            dummy_repository_package_factory("nose", "1.3.0", 1)
        ]

        with mock.patch("enstaller.main.Enpkg.execute") as m:
            error = OSError()
            error.errno = errno.EACCES
            m.side_effect = error
            enpkg = create_prefix_with_eggs(config, self.prefix, [], remote_entries)
            with self.assertRaises(OSError):
                install_req(enpkg, config, "nose", FakeOptions())

    def test_simple_install_pypi(self):
        self.maxDiff = None

        # Given
        entry = dummy_repository_package_factory("nose", "1.3.0", 1)
        entry.product = "pypi"
        remote_entries = [entry]
        r_message = textwrap.dedent("""\
        The following packages/requirements are coming from the PyPi repo:

        nose

        The PyPi repository which contains >10,000 untested ("as is")
        packages. Some packages are licensed under GPL or other licenses
        which are prohibited for some users. Dependencies may not be
        provided. If you need an updated version or if the installation
        fails due to unmet dependencies, the Knowledge Base article
        Installing external packages into Canopy Python
        (https://support.enthought.com/entries/23389761) may help you with
        installing it.

        Are you sure that you wish to proceed?  (y/[n])
        """)

        # When
        with mock_print() as mocked_print:
            with mock_raw_input("yes"):
                with mock.patch("enstaller.main.Enpkg.execute") as m:
                    enpkg = create_prefix_with_eggs(Configuration(),
                                                    self.prefix, [],
                                                    remote_entries)
                    install_req(enpkg, Configuration(), "nose", FakeOptions())

        # Then
        self.assertMultiLineEqual(mocked_print.value, r_message)
        m.assert_called_with([('fetch', 'nose-1.3.0-1.egg'),
                              ('install', 'nose-1.3.0-1.egg')])


class TestCustomConfigPath(unittest.TestCase):
    def setUp(self):
        self.prefix = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.prefix)

    @responses.activate
    def test_simple(self):
        # Given
        path = os.path.join(self.prefix, "enstaller.yaml")
        responses.add(responses.POST, "http://acme.com/api/v0/json/auth/tokens/auth",
                      body=json.dumps({"token": "dummy token"}))

        with open(path, "wt") as fp:
            fp.write(textwrap.dedent("""\
                    store_url: "http://acme.com"
                    authentication:
                      kind: "simple"
                      username: "foo@acme.com"
                      password: "bar"
            """))

        # When
        # Then No exception
        main(["-s", "numpy", "-c", path])


class TestEnstallerComparableVersion(unittest.TestCase):
    def setUp(self):
        self.prefix = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.prefix)

    def test_no_egg_install(self):
        # Given
        prefix = self.prefix
        package_name = "enstaller"
        r_version = EnpkgVersion.from_upstream_and_build(enstaller.__version__,
                                                         1)

        # When
        version = _get_enstaller_comparable_version(prefix, package_name)

        # Then
        self.assertEqual(version, r_version)

    def test_egg_install(self):
        # ensure that we use the build number from installed metadata if
        # version and enstaller.__version__ are the same

        # Given
        prefix = self.prefix
        package_name = "enstaller"
        version_string = "2.7.6"
        build = 4
        r_version = EnpkgVersion.from_upstream_and_build(version_string, build)

        json_dict = {
            "arch": None,
            "build": build,
            "ctime": "Mon Oct 20 15:49:19 2014",
            "hook": False,
            "key": "enstaller-{0}-1.egg".format(version_string),
            "name": "enstaller",
            "osdist": None,
            "packages": [],
            "platform": None,
            "python": "2.7",
            "type": "egg",
            "version": version_string,
        }

        meta_info_path = meta_info_from_prefix(prefix, package_name)
        ensure_dir(meta_info_path)
        with open(meta_info_path, "wt") as fp:
            json.dump(json_dict, fp)

        # When
        with mock.patch("enstaller.__version__", version_string):
            version = _get_enstaller_comparable_version(prefix, package_name)

        # Then
        self.assertEqual(version, r_version)

    def test_egg_install_different_versions(self):
        # ensure that we use the build number from installed metadata if
        # version and enstaller.__version__ are the same

        # Given
        prefix = self.prefix
        package_name = "enstaller"
        version_string = "2.7.6"
        build = 4

        json_dict = {
            "arch": None,
            "build": build,
            "ctime": "Mon Oct 20 15:49:19 2014",
            "hook": False,
            "key": "enstaller-{0}-1.egg".format(version_string),
            "name": "enstaller",
            "osdist": None,
            "packages": [],
            "platform": None,
            "python": "2.7",
            "type": "egg",
            "version": version_string,
        }
        r_version = EnpkgVersion.from_upstream_and_build("4.8.0", 1)

        meta_info_path = meta_info_from_prefix(prefix, package_name)
        ensure_dir(meta_info_path)
        with open(meta_info_path, "wt") as fp:
            json.dump(json_dict, fp)

        # When
        with mock.patch("enstaller.__version__", "4.8.0"):
            version = _get_enstaller_comparable_version(prefix, package_name)

        # Then
        self.assertEqual(version, r_version)


class TestConfigurationSetup(unittest.TestCase):
    @responses.activate
    def test_ensure_authenticated_config(self):
        # Given
        r_message = textwrap.dedent("""\
            Could not authenticate as 'nono'
            Please check your credentials/configuration and try again
            (original error is: 'Authentication error: Invalid user login.').


            You can change your authentication details with 'enpkg --userpass'.
        """)

        store_url = "https://acme.com"
        responses.add(responses.GET, store_url + "/accounts/user/info/",
                      body=json.dumps(R_JSON_NOAUTH_RESP))

        config = Configuration()
        config.update(store_url=store_url, auth=("nono", "le petit robot"))

        session = Session.from_configuration(config)

        # When/Then
        with mock_print() as m:
            with self.assertRaises(SystemExit) as e:
                ensure_authenticated_config(config, "", session)

        self.assertEqual(exception_code(e), -1)
        self.assertMultiLineEqual(m.value, r_message)


@mock.patch("enstaller.main.install_req")
class TestMainYamlConfig(unittest.TestCase):
    def setUp(self):
        self.prefix = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.prefix)

    @mock_index({})
    def test_home_expand(self, install_req):
        # Given
        path = "~/config.yaml"
        args = ["--config-path=~/config.yaml", "foo"]

        r_path = os.path.expanduser(path)

        # When
        with mock.patch("enstaller.main.Configuration.from_yaml_filename",
                        return_value=Configuration()) \
                as mocked_factory:
            with self.assertRaises(SystemExit) as exc:
                main(args)
            self.assertEqual(exception_code(exc), -1)

        # Then
        mocked_factory.assert_called_with(r_path)

    @mock_index({})
    def test_non_existing_config_path(self, install_req):
        # Given
        args = ["--config-path=config.yaml", "foo"]

        # When/Then
        with self.assertRaises(SystemExit) as exc:
            main(args)
        self.assertEqual(exception_code(exc), -1)

    @mock_index({})
    def test_config_path_missing_auth(self, install_req):
        # Given
        path = os.path.join(self.prefix, "config.yaml")
        with open(path, "wt") as fp:
            fp.write("")

        args = ["--config-path=" + path, "foo"]

        r_msg = "Authentication missing from {0!r}\n".format(path)

        # When
        with mock_print() as m:
            with self.assertRaises(SystemExit) as exc:
                main(args)

        # Then
        self.assertEqual(exception_code(exc), -1)
        self.assertEqual(m.value, r_msg)


@mock.patch("enstaller.main.install_req")
@authenticated_config
class TestMain(unittest.TestCase):
    def setUp(self):
        self.prefix = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.prefix)

    @mock_index({})
    def test_setup_proxy(self, install_req):
        # Given
        args = ["--proxy=http://acme.com:3128", "foo"]

        # When
        with mock.patch("enstaller.main.setup_proxy_or_die") as m:
            main(args)

        # Then
        m.assert_called()
        self.assertEqual(m.call_args[0][1], "http://acme.com:3128")

    @mock_index({})
    def test_user_valid_config(self, install_req):
        # Given
        args = ["--user", "foo"]

        # When
        with mock.patch("enstaller.main.check_prefixes") as m:
            main(args)

        # Then
        m.assert_called_with([site.USER_BASE, os.path.normpath(sys.prefix)])

    @mock_index({})
    def test_user_invalid_config(self, install_req):
        # Given
        r_msg = "Using the --user option, but your PYTHONPATH is not setup " \
                "accordingly"

        args = ["--user", "foo"]

        # When/Then
        with self.assertWarnsRegex(Warning, r_msg):
            main(args)

    @mock_index({})
    def test_max_retries(self, install_req):
        # Given
        args = ["--max-retries", "42"]

        # When
        with mock.patch("enstaller.main.dispatch_commands_with_enpkg") as m:
            main(args)
        config = m.call_args[0][2]

        # Then
        m.assert_called()
        self.assertEqual(config.max_retries, 42)

    @mock_index({})
    def test_quiet(self, install_req):
        # Given
        args = ["foo"]

        # When
        with mock.patch("sys.stdout", new=StringIO()) as m:
            main(args)

        # Then
        self.assertNotEqual(m.getvalue(), "")

        # Given
        args = ["foo", "--quiet"]

        # When
        with mock.patch("sys.stdout", new=StringIO()) as m:
            main(args)

        # Then
        self.assertEqual(m.getvalue(), "")

    @mock_index({})
    def test_verbose_flag(self, install_req):
        # Given
        args = ["foo"]

        # When
        with mock.patch("enstaller.logging.basicConfig") as m:
            main(args)

        # Then
        m.assert_called()
        self.assertEqual(m.call_args[1]["level"], logging.WARN)

        # Given
        args = ["foo", "-v"]

        # When
        with mock.patch("enstaller.logging.basicConfig") as m:
            main(args)

        # Then
        m.assert_called()
        self.assertEqual(m.call_args[1]["level"], logging.INFO)

        # Given
        args = ["foo", "-vv"]

        # When
        with mock.patch("enstaller.main.http_client.HTTPConnection") \
                as fake_connection:
            with mock.patch("enstaller.logging.basicConfig") as m:
                main(args)

        # Then
        fake_connection.assert_not_called()

        m.assert_called()
        self.assertEqual(m.call_args[1]["level"], logging.DEBUG)

        # Given
        args = ["foo", "-vvv"]

        # When
        with mock.patch("enstaller.main.http_client.HTTPConnection") \
                as fake_connection:
            with mock.patch("enstaller.logging.basicConfig") as m:
                main(args)

        # Then
        fake_connection.assert_called()
        self.assertEqual(fake_connection.debuglevel, 1)

        m.assert_called()
        self.assertEqual(m.call_args[1]["level"], logging.DEBUG)

    @authenticated_config
    @mock_index({})
    def test_no_autoupdate(self, install_req):
        # Given
        args = ["foo"]

        # When
        with mock.patch("enstaller.main.update_enstaller") as m:
            main(args)

        # Then
        self.assertTrue(m.called)

        # Given
        args = ["--no-autoupdate", "foo"]

        # When
        with mock.patch("enstaller.main.update_enstaller") as m:
            main(args)

        # Then
        self.assertFalse(m.called)

        # Given
        args = ["foo"]

        config = Configuration()
        config.update(autoupdate=False, auth=("dummy", "dummy"))

        mock_config = mock.Mock()
        mock_config.return_value = config
        mock_config.from_file.return_value = config

        # When
        with mock.patch("enstaller.main.Configuration", mock_config):
            with mock.patch("enstaller.main.update_enstaller") as m:
                main(args)

        # Then
        self.assertFalse(m.called)



from egginst.vendor.six.moves import unittest

from ..enpkg import EnpkgVersion
from ..pep386_workaround import PEP386WorkaroundVersion


class TestEnpkgVersionParsing(unittest.TestCase):
    def test_hashing(self):
        # Given
        s = "1.3.0-1"

        # When
        v1 = EnpkgVersion.from_string(s)
        v2 = EnpkgVersion.from_string(s)

        # Then
        self.assertEqual(v1, v2)
        self.assertEqual(hash(v1), hash(v2))

    def test_from_string_valid(self):
        # Given
        s = "1.3.0-1"

        # When
        version = EnpkgVersion.from_string(s)

        # Then
        self.assertEqual(version.upstream,
                         PEP386WorkaroundVersion.from_string("1.3.0"))
        self.assertEqual(version.build, 1)

        # Given
        s = "1.2.0-3"

        # When
        version = EnpkgVersion.from_string(s)

        # Then
        self.assertEqual(version.upstream,
                         PEP386WorkaroundVersion.from_string("1.2.0"))
        self.assertEqual(version.build, 3)

        # Given
        s = "1.2-3"

        # When
        version = EnpkgVersion.from_string(s)

        # Then
        self.assertEqual(version.upstream,
                         PEP386WorkaroundVersion.from_string("1.2.0"))
        self.assertEqual(version.build, 3)

    def test_from_string_invalid(self):
        # Given
        s = "1.3.0"

        # When/Then
        with self.assertRaises(ValueError):
            EnpkgVersion.from_string(s)

        # Given
        s = "1.3.0-a"

        # When/Then
        with self.assertRaises(ValueError):
            EnpkgVersion.from_string(s)

    def test_cannot_compare(self):
        # Given
        left = EnpkgVersion.from_string("1.3.0-1")
        right = PEP386WorkaroundVersion.from_string("1.3.0")

        # When/Then
        with self.assertRaises(TypeError):
            left == right

        # When/Then
        with self.assertRaises(TypeError):
            left < right

        # When/Then
        with self.assertRaises(TypeError):
            left > right

        # When/Then
        with self.assertRaises(TypeError):
            left <= right

        # When/Then
        with self.assertRaises(TypeError):
            left >= right


class TestEnpkgVersionComparison(unittest.TestCase):
    def test_equal(self):
        # Given
        left = EnpkgVersion.from_string("1.2.0-1")
        right = EnpkgVersion.from_string("1.2.0-1")

        # When/Then
        self.assertEqual(left, right)

        # Given
        left = EnpkgVersion.from_string("1.2.0-1")
        right = EnpkgVersion.from_string("1.2-1")

        # When/Then
        self.assertEqual(left, right)

        # Given
        left = EnpkgVersion.from_string("1.2.0-1")
        right = EnpkgVersion.from_string("1.2.0-2")

        # When/Then
        self.assertNotEqual(left, right)

        # Given
        left = EnpkgVersion.from_string("1.1.0-1")
        right = EnpkgVersion.from_string("1.2.0-1")

        # When/Then
        self.assertNotEqual(left, right)

    def test_less_than(self):
        # Given
        left = EnpkgVersion.from_string("1.2.0-1")
        right = EnpkgVersion.from_string("1.2.0-1")

        # When/Then
        self.assertTrue(left <= right)
        self.assertTrue(left >= right)
        self.assertFalse(left < right)
        self.assertFalse(left > right)

        # Given
        left = EnpkgVersion.from_string("1.2.0-1")
        right = EnpkgVersion.from_string("1.2.0-2")

        # When/Then
        self.assertTrue(left <= right)
        self.assertTrue(left < right)
        self.assertFalse(left >= right)
        self.assertFalse(left > right)

        # Given
        left = EnpkgVersion.from_string("1.2.0-2")
        right = EnpkgVersion.from_string("1.2.1-1")

        # When/Then
        self.assertTrue(left <= right)
        self.assertTrue(left < right)
        self.assertFalse(left >= right)
        self.assertFalse(left > right)


class TestEnpkgVersionMisc(unittest.TestCase):
    def test_string(self):
        # Given
        r_version_string = "1.3.0-1"
        v = EnpkgVersion.from_string(r_version_string)

        # When
        version_string = str(v)

        # Then
        self.assertEqual(version_string, r_version_string)



import os
import shutil
import sys
import tempfile

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

from machotools import rewriter_factory

from egginst.main import EggInst
from egginst.object_code import (_compute_targets, _find_lib, _fix_object_code,
                                 get_object_type)
from egginst import _zipfile

from .common import (DUMMY_EGG_WITH_INST_TARGETS, FILE_TO_RPATHS,
                     LEGACY_PLACEHOLD_FILE_RPATH, NOLEGACY_RPATH_FILE,
                     MACHO_ARCH_TO_FILE,
                     PYEXT_WITH_LEGACY_PLACEHOLD_DEPENDENCY, PYEXT_DEPENDENCY)


class TestObjectCode(unittest.TestCase):
    def setUp(self):
        self.prefix = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.prefix)

    def test_get_object_type(self):
        self.assertEqual(get_object_type(MACHO_ARCH_TO_FILE["x86"]), "MachO-i386")
        self.assertEqual(get_object_type(MACHO_ARCH_TO_FILE["amd64"]), "MachO-x86_64")

        self.assertEqual(get_object_type("dummy_no_exist"), None)
        self.assertEqual(get_object_type(__file__), None)

    def test_fix_object_code_legacy_macho(self):
        """
        Test that we handle correctly our legacy egg with the /PLACHOLD * 20 hack.
        """
        copy = os.path.join(self.prefix, "foo.dylib")
        shutil.copy(LEGACY_PLACEHOLD_FILE_RPATH, copy)

        _fix_object_code(copy, [self.prefix])
        rpaths = rewriter_factory(copy).rpaths

        self.assertEqual(rpaths, [self.prefix])

    def test_fix_object_code_wo_legacy_macho(self):
        """
        Test that we handle correctly egg *without* the /PLACHOLD (i.e. we
        don't touch them).
        """
        r_rpaths = FILE_TO_RPATHS[NOLEGACY_RPATH_FILE]
        copy = os.path.join(self.prefix, "foo.dylib")
        shutil.copy(NOLEGACY_RPATH_FILE, copy)

        _fix_object_code(copy, self.prefix)
        rpaths = rewriter_factory(copy).rpaths

        self.assertEqual(rpaths, r_rpaths)

    def test_fix_object_code_with_targets(self):
        """
        Test we handle the targets.dat hack correctly in fix_object_code.

        Some of our eggs use /PLACEHOLD in the LC_LOAD_DYLIB command (i.e.
        inside the dependency names). Those are rewritten by finding the
        dependency in the installed egg first.

        We're ensuring here that those are also rewritten correctly.
        """
        # targets.dat hack is handled in the function that expects an egg, but
        # we don't want to deal with an egg. Instead, we emulate the
        # targets.dat hack by patching object_code._targets with mock
        pyext_dependency_dir = os.path.join("lib", "foo-4.2")

        installed_pyext_dependency = os.path.join(self.prefix,
                                                  pyext_dependency_dir,
                                                  os.path.basename(PYEXT_DEPENDENCY))
        installed_pyext_dependency_dir = os.path.dirname(installed_pyext_dependency)
        targets = [self.prefix, installed_pyext_dependency_dir]

        installed_pyext = os.path.join(self.prefix,
                                       os.path.basename(PYEXT_WITH_LEGACY_PLACEHOLD_DEPENDENCY))
        shutil.copy(PYEXT_WITH_LEGACY_PLACEHOLD_DEPENDENCY, installed_pyext)

        os.makedirs(installed_pyext_dependency_dir)
        shutil.copy(PYEXT_DEPENDENCY, installed_pyext_dependency_dir)

        _fix_object_code(installed_pyext, targets)
        deps = set(rewriter_factory(installed_pyext).dependencies)

        self.assertTrue(installed_pyext_dependency in deps)

    @unittest.skipIf(sys.platform == "win32", "This feature is not used on windows.")
    def test_find_lib_with_targets(self):
        def _compute_target_list(path, d):
            # FIXME: we need this hack as the internal EggInst zipfile
            # object is only available within an EggInst.install call.
            egg_inst = EggInst(path, d)
            egg_inst.z = zp

            return _compute_targets(egg_inst.iter_targets(), egg_inst.prefix)

        # Given
        with _zipfile.ZipFile(DUMMY_EGG_WITH_INST_TARGETS) as zp:
            egg_inst = EggInst(DUMMY_EGG_WITH_INST_TARGETS, self.prefix)
            egg_inst.install()

            targets = _compute_target_list(DUMMY_EGG_WITH_INST_TARGETS, self.prefix)

            path = "libfoo.dylib"
            r_found_lib = os.path.join(self.prefix, "lib", "foo-4.2", path)

            # When
            found_lib = _find_lib(path, targets)

            # Then
            self.assertEqual(found_lib, r_found_lib)



import contextlib
import os.path
import shutil
import tempfile
import textwrap

import mock

from okonomiyaki.errors import OkonomiyakiError
from okonomiyaki.file_formats import EggMetadata
from okonomiyaki.platforms import EPDPlatform

from egginst._compat import assertCountEqual
from egginst._zipfile import ZipFile
from egginst.eggmeta import info_from_z
from egginst.tests.common import (
    DUMMY_EGG, STANDARD_EGG, LEGACY_EGG_INFO_EGG, STANDARD_EGG_WITH_EXT,
    NOSE_1_2_1, MKL_10_3,
)
from egginst.vendor.six.moves import unittest

from enstaller.errors import EnstallerException
from enstaller.tools.repack import InvalidVersion, main, repack


@contextlib.contextmanager
def chdir(d):
    old = os.getcwd()
    os.chdir(d)
    yield old
    os.chdir(old)


class TestRepack(unittest.TestCase):
    def setUp(self):
        self.prefix = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.prefix)

    def test_repack_default_platform_unsupported(self):
        # Given
        source = os.path.join(self.prefix, "nose-1.2.1-py2.7.egg")
        shutil.copy(STANDARD_EGG, source)

        target = os.path.join(self.prefix, "nose-1.2.1-1.egg")

        # When/Then
        mocked = "enstaller.tools.repack.EPDPlatform.from_running_system"
        with mock.patch(mocked, side_effect=OkonomiyakiError()):
            with self.assertRaises(EnstallerException):
                repack(source, 1)

        # Then
        self.assertFalse(os.path.exists(target))

    def test_repack_default_platform(self):
        # Given
        source = os.path.join(self.prefix, "nose-1.2.1-py2.7.egg")
        shutil.copy(STANDARD_EGG, source)

        target = os.path.join(self.prefix, "nose-1.2.1-1.egg")

        # When
        mocked = "enstaller.tools.repack.EPDPlatform.from_running_system"
        platform = EPDPlatform.from_epd_string("rh5-32")
        with mock.patch(mocked, return_value=platform):
            repack(source, 1)

        # Then
        self.assertTrue(os.path.exists(target))
        with ZipFile(target) as zp:
            spec = info_from_z(zp)
        self.assertEqual(spec["arch"], "x86")

    def test_unsupported_format(self):
        # Given
        source = os.path.join(self.prefix, "nose-1.2.1-1.egg")
        shutil.copy(STANDARD_EGG, source)

        # When/Then
        with self.assertRaises(EnstallerException) as e:
            repack(source, 1, "rh5-64")
        self.assertTrue(str(e.exception).startswith("Unrecognized format"))

    def test_refuse_inplace(self):
        # Given
        target = os.path.join(self.prefix, "dummy-1.0.1-1.egg")
        source = target
        r_msg = "source and repack-ed egg are the same"

        shutil.copy(DUMMY_EGG, source)

        # When/Then
        with self.assertRaises(EnstallerException) as e:
            repack(source, 1, "rh5-64")

        self.assertTrue(str(e.exception).startswith(r_msg))

    def test_setuptools_egg_with_ext(self):
        # Given
        source = os.path.join(self.prefix, os.path.basename(STANDARD_EGG_WITH_EXT))
        shutil.copy(STANDARD_EGG_WITH_EXT, source)

        target = os.path.join(self.prefix, "PyYAML-3.11-1.egg")

        # When
        repack(source, 1, "rh5-64")

        # Then
        self.assertTrue(os.path.exists(target))
        with ZipFile(target) as zp:
            zp.read("EGG-INFO/spec/depend")

    def test_setuptools_egg_with_ext_without_platform(self):
        # Given
        r_msg = "Platform-specific egg detected (platform string is " \
                "'linux-x86_64'), you *must* specify the platform."
        source = os.path.join(self.prefix, os.path.basename(STANDARD_EGG_WITH_EXT))
        shutil.copy(STANDARD_EGG_WITH_EXT, source)

        # When/Then
        with self.assertRaises(EnstallerException) as exc:
            repack(source, 1)

        self.assertEqual(str(exc.exception), r_msg)

    def test_simple_setuptools_egg(self):
        # Given
        source = os.path.join(self.prefix, os.path.basename(STANDARD_EGG))
        shutil.copy(STANDARD_EGG, source)

        target = os.path.join(self.prefix, "Jinja2-2.6-1.egg")

        # When
        repack(source, 1, "rh5-64")

        # Then
        self.assertTrue(os.path.exists(target))
        with ZipFile(target) as zp:
            zp.read("jinja2/__init__.py")

    def test_simple_enthought_egg(self):
        # Given
        source = os.path.join(self.prefix, os.path.basename(NOSE_1_2_1))
        shutil.copy(NOSE_1_2_1, source)

        target = os.path.join(self.prefix, "nose-1.2.1-2.egg")

        # When
        repack(source, 2, "rh5-64")

        # Then
        self.assertTrue(os.path.exists(target))

    def test_dependencies(self):
        # Given
        egg = LEGACY_EGG_INFO_EGG
        source = os.path.join(self.prefix, os.path.basename(egg))
        shutil.copy(egg, source)
        r_runtime_dependencies = EggMetadata.from_egg(egg).runtime_dependencies

        target = os.path.join(self.prefix, egg)

        # When
        repack(source, 11, "rh5-64")

        # Then
        self.assertTrue(os.path.exists(target))
        metadata = EggMetadata.from_egg(target)
        self.assertEqual(metadata.runtime_dependencies, r_runtime_dependencies)

    def test_enthought_name_upper_case(self):
        # Given
        source = os.path.join(self.prefix,
                              os.path.basename(MKL_10_3))
        shutil.copy(MKL_10_3, source)

        target = os.path.join(self.prefix, "MKL-10.3-11.egg")

        # When
        repack(source, 11, "rh5-64")

        # Then
        self.assertTrue(os.path.exists(target))
        metadata = EggMetadata.from_egg(target)
        self.assertEqual(metadata.egg_basename, "MKL")

    def test_endist_metadata_simple(self):
        # Given
        source = os.path.join(self.prefix, os.path.basename(NOSE_1_2_1))
        shutil.copy(NOSE_1_2_1, source)

        target = os.path.join(self.prefix, "babar-1.2.1-2.egg")
        endist = os.path.join(self.prefix, "endist.dat")
        with open(endist, "w") as fp:
            data = textwrap.dedent("""\
            packages = ["foo"]

            name = "babar"
            """)
            fp.write(data)

        # When
        with chdir(self.prefix):
            repack(source, 2, "rh5-64")

        # Then
        self.assertTrue(os.path.exists(target))
        with ZipFile(target) as zp:
            info = info_from_z(zp)
        assertCountEqual(self, info["packages"], ["foo"])
        assertCountEqual(self, info["name"], "babar")

    def test_endist_add_files_simple(self):
        # Given
        source = os.path.join(self.prefix, os.path.basename(NOSE_1_2_1))
        shutil.copy(NOSE_1_2_1, source)

        target = os.path.join(self.prefix, "nose-1.2.1-2.egg")
        endist = os.path.join(self.prefix, "endist.dat")
        with open(endist, "w") as fp:
            data = textwrap.dedent("""\
            packages = ["foo"]

            add_files = [(".", "foo*", "EGG-INFO")]
            """)
            fp.write(data)
        fubar = os.path.join(self.prefix, "foo.txt")
        with open(fubar, "w") as fp:
            fp.write("babar")

        # When
        with chdir(self.prefix):
            repack(source, 2, "rh5-64")

        # Then
        self.assertTrue(os.path.exists(target))
        with ZipFile(target) as zp:
            data = zp.read("EGG-INFO/foo.txt").decode("utf8")
        self.assertEqual(data, "babar")

    def test_endist_unsupported_key(self):
        # Given
        source = os.path.join(self.prefix, os.path.basename(NOSE_1_2_1))
        shutil.copy(NOSE_1_2_1, source)

        endist = os.path.join(self.prefix, "endist.dat")
        with open(endist, "w") as fp:
            data = "app_icon_file = 'fubar.ico'"
            fp.write(data)

        # When/Then
        with chdir(self.prefix):
            with self.assertRaises(NotImplementedError):
                repack(source, 2, "rh5-64")

    def test_invalid_version(self):
        # Given an invalid version with a suggestion
        source = os.path.join(self.prefix, "nose-1.2.1dev-py2.7.egg")
        shutil.copy(NOSE_1_2_1, source)

        r_msg = ("The given version '1.2.1dev' does not follow PEP 386."
                 " Please change the egg version to a valid format \(e.g."
                 " '1.2.1.dev0'\).")

        # When/Then
        with chdir(self.prefix):
            with self.assertRaisesRegexp(InvalidVersion, r_msg):
                repack(source, 1, "rh5-64")

        # Given a valid version w/o any suggestion
        source = os.path.join(self.prefix, "nose-1.2.1ddv-py2.7.egg")
        shutil.copy(NOSE_1_2_1, source)

        r_msg = ("The given version '1.2.1ddv' does not follow PEP 386."
                 " Please change the egg version to a valid format.")

        # When/Then
        with chdir(self.prefix):
            with self.assertRaisesRegexp(InvalidVersion, r_msg):
                repack(source, 1, "rh5-64")


class TestRepackMain(unittest.TestCase):
    def test_help(self):
        # Given
        args = ["-h"]

        # When
        with self.assertRaises(SystemExit) as e:
            with mock.patch("enstaller.tools.repack.repack") as repack:
                main(args)

        # Then
        self.assertFalse(repack.called)
        self.assertEqual(e.exception.code, 0)

    def test_usual_args(self):
        # Given
        args = ["nose-1.3.0-py2.7.egg", "-b", "3", "-a", "win-32"]

        # When
        with mock.patch("enstaller.tools.repack.repack") as repack:
            main(args)

        # Then
        self.assertTrue(repack.called)
        self.assertTrue(repack.called_with("nose-1.3.0-py2.7.egg", "-b",
                                           "3", "-a", "win-32"))



# FIXME: those are in egginst to avoid egginst<->enstaller circular import
class EnstallerException(Exception):
    pass


class InvalidChecksum(EnstallerException):
    def __init__(self, filename, expected_checksum, actual_checksum):
        template = "Checksum mismatch for {0!r}: received {1!r} " \
                   "(expected {2!r})"
        self.msg = template.format(filename, actual_checksum,
                                   expected_checksum)

    def __str__(self):
        return self.msg


class ProcessCommunicationError(EnstallerException):
    pass


class ConnectionError(EnstallerException):
    pass


class InvalidPythonPathConfiguration(EnstallerException):
    pass


class InvalidConfiguration(EnstallerException):
    pass


class InvalidFormat(InvalidConfiguration):
    def __init__(self, message, lineno=None, col_offset=None):
        self.message = message
        self.lineno = lineno
        self.col_offset = col_offset

    def __str__(self):
        return self.message


class AuthFailedError(EnstallerException):
    def __init__(self, *args):
        super(AuthFailedError, self).__init__(*args)
        if len(args) > 1:
            self.original_exception = args[1]
        else:
            self.original_exception = None


class EnpkgError(EnstallerException):
    # FIXME: why is this a class-level attribute ?
    req = None


class NoSuchPackage(EnstallerException):
    pass


class SolverException(EnstallerException):
    pass


class NoPackageFound(SolverException):
    """Exception thrown if no egg can be found for the given requirement."""

    def __init__(self, msg, requirement):
        super(NoPackageFound, self).__init__(msg)
        self.requirement = requirement


class UnavailablePackage(EnstallerException):
    """Exception thrown when a package is not available for a given
    subscription level."""

    def __init__(self, requirement):
        self.requirement = requirement


class MissingDependency(SolverException):
    """Exception thrown when a dependency for package is not available."""

    def __init__(self, msg, requester, requirement):
        super(MissingDependency, self).__init__(msg)
        self.requirement = requirement
        self.requester = requester

EXIT_ABORTED = 130
